import inspect
import json
import os
import time
import uuid
from contextlib import contextmanager
from typing import TYPE_CHECKING, Dict, Iterator, Optional

from opentelemetry import trace
from opentelemetry.trace import Status, StatusCode
from opentelemetry.util import types

if TYPE_CHECKING:
    from common.service.oss.base_oss import BaseOSSService

# from common.otlp.sid import sid_generator2 as sid_gen
from common.otlp import sid as sid_module
from common.otlp.log_trace.node_log import NodeLog
from common.otlp.trace.trace import SpanLevel

# from xf_langflow.otlp.log_trace.node_log import NodeLog

SPAN_SIZE_LIMIT = 10 * 1024


class Span:
    sid: str
    app_id: str
    uid: str
    chat_id: str

    def __init__(
        self,
        app_id: str = "",
        uid="",
        chat_id="",
        oss_service: Optional["BaseOSSService"] = None,
    ):
        self.app_id = app_id
        self.uid = uid
        self.chat_id = chat_id
        if sid_module.sid_generator2 is None:
            raise Exception("sid_generator2 is not initialized")
        self.sid = sid_module.sid_generator2.gen()
        self.tracer = trace.get_tracer(os.getenv("SERVICE_NAME", "service_trace"))
        self.oss_service = oss_service

    @contextmanager
    def start(
        self,
        func_name: str = "",
        add_source_function_name: bool = False,
        attributes: Optional[dict] = None,
        trace_context: Optional[Dict] = None,
    ) -> Iterator["Span"]:
        """
        开始一个span
        :param func_name:                   方法名
        :param add_source_function_name:    是否添加原始调用方法名
        :param attributes:                  属性
        :param trace_context:               trace上下文
        :return:
        """
        if not func_name:
            func_name = self._get_source_function_name()
        if func_name and add_source_function_name:
            func_name = func_name + "::" + self._get_source_function_name()
        default_attr = {
            "sid": self.sid,
            "app_id": self.app_id,
            "uid": self.uid,
            "chat_id": self.chat_id,
            "span_version": "1.0.0",
        }
        if attributes:
            default_attr.update(attributes)

        context = None
        if trace_context:
            from common.otlp.trace.trace import Trace as CTrace

            context = CTrace.extract_context(trace_context)

        with self.tracer.start_as_current_span(
            func_name, context=context, attributes=default_attr
        ):
            yield self

    def _get_source_function_name(self):
        cf = inspect.currentframe()
        if cf:
            frame = cf.f_back
            if frame:
                frame = frame.f_back
                if frame:
                    frame = frame.f_back
                    if frame:
                        f_code = frame.f_code
                        if f_code:
                            return f_code.co_name
        return "unknown"

    def set_attribute(self, key: str, value, node_log: Optional[NodeLog] = None):
        """
        设置属性
        :param node_log:
        :param key:
        :param value:
        :return:
        """
        self.get_otlp_span().set_attribute(key, value)
        if node_log:
            node_log.add_info_log(f"set attribute: {key}={value}")

    def set_status(self, status: Status):
        """
        设置状态
        :param status:
        :return:
        """
        self.get_otlp_span().set_status(status)

    def set_attributes(self, attributes: dict, node_log: Optional[NodeLog] = None):
        """
        设置属性
        :param node_log:
        :param attributes:
        :return:
        """
        self.get_otlp_span().set_attributes(attributes)
        if node_log:
            node_log.add_info_log(f"set attributes: {attributes}")

    def set_code(self, code: int, node_log: Optional[NodeLog] = None):
        """
        设置状态码
        :param node_log:
        :param code:
        :return:
        """
        self.set_attribute("code", code, node_log)

    def get_otlp_span(self):
        return trace.get_current_span()

    def record_exception(
        self,
        ex: Exception,
        attributes: types.Attributes = None,
        node_log: Optional[NodeLog] = None,
    ):
        """
        记录异常
        :param node_log:
        :param attributes:
        :param ex:
        :return:
        """
        self.get_otlp_span().record_exception(
            ex, attributes=attributes, timestamp=int(int(round(time.time() * 1000)))
        )
        self.set_status(Status(StatusCode.ERROR))
        if node_log:
            node_log.add_error_log(f"{str(ex)}")
            node_log.set_end()
            node_log.running_status = False

    def add_event(
        self,
        name: str,
        attributes: Optional[types.Attributes] = None,
        timestamp: Optional[int] = None,
        node_log: Optional[NodeLog] = None,
    ):
        """
        添加事件，如日志
        :param node_log:
        :param name:
        :param attributes:
        :param timestamp:
        :return:
        """
        self.get_otlp_span().add_event(name, attributes=attributes, timestamp=timestamp)
        if node_log and attributes:
            node_log.add_info_log(f"{name}={attributes}")

    def add_info_event(self, value: str, node_log: Optional[NodeLog] = None):
        """
        添加INFO事件
        :param node_log: 节点链路日志
        :param value:
        :return:
        """
        value_bytes = value.encode("utf-8")
        if len(value_bytes) >= SPAN_SIZE_LIMIT:
            try:
                if self.oss_service is not None:
                    trace_link = self.oss_service.upload_file(
                        f"{str(uuid.uuid4())}", value_bytes
                    )
                    value = f"trace_link: {trace_link}"
            except Exception:
                value = "日志内容过大，上传s3存储时失败"
        self.get_otlp_span().add_event("INFO", attributes={"INFO LOG": value})
        if node_log:
            node_log.add_info_log(f"{value}")

    def add_info_events(
        self,
        attributes: Optional[types.Attributes] = None,
        timestamp: Optional[int] = None,
        node_log: Optional[NodeLog] = None,
    ):
        """
        添加INFO事件
        :param node_log:
        :param attributes:
        :param timestamp:
        :return:
        """
        value_bytes = json.dumps(attributes, ensure_ascii=False).encode("utf-8")
        if len(value_bytes) >= SPAN_SIZE_LIMIT:
            try:
                if self.oss_service is not None:
                    trace_link = self.oss_service.upload_file(
                        f"{str(uuid.uuid4())}", value_bytes
                    )
                    attributes = {"trace_link": trace_link}
            except Exception:
                attributes = {"error": "日志内容过大，上传s3存储时失败"}
        self.get_otlp_span().add_event(
            SpanLevel.INFO.value, attributes=attributes, timestamp=timestamp
        )
        if node_log:
            node_log.add_info_log(f"{attributes}")

    def add_error_event(self, value, node_log: Optional[NodeLog] = None):
        """

        :param value:
        :param node_log:
        :return:
        """
        self.set_attribute("error", True)
        self.get_otlp_span().add_event(
            SpanLevel.ERROR.value, attributes={"ERROR LOG": value}
        )
        if node_log:
            node_log.add_error_log(f"{value}")
            node_log.set_end()
            node_log.running_status = False

    def add_error_events(
        self,
        attributes: Optional[types.Attributes] = None,
        timestamp: Optional[int] = None,
        node_log: Optional[NodeLog] = None,
    ):
        """

        :param node_log:
        :param attributes:
        :param timestamp:
        :return:
        """
        self.get_otlp_span().add_event(
            SpanLevel.ERROR.value, attributes=attributes, timestamp=timestamp
        )
        if node_log:
            node_log.add_error_log(f"{attributes}")
            node_log.set_end()
            node_log.running_status = False



import os

from loguru import logger

from common.service.base import ServiceFactory
from common.service.cache.base_cache import RedisModel
from common.service.cache.redis_cache import RedisCache


class CacheServiceFactory(ServiceFactory):
    def __init__(self):
        super().__init__(RedisCache)

    def create(self) -> RedisCache:
        logger.debug("Creating Redis cache")
        redis_cluster_addr = os.getenv("REDIS_CLUSTER_ADDR", "")
        redis_addr = os.getenv("REDIS_ADDR", "")
        if not redis_cluster_addr and not redis_addr:
            raise RuntimeError("REDIS_CLUSTER_ADDR or REDIS_ADDR must be set")
        redis_cache = RedisCache(
            expiration_time=os.getenv("REDIS_EXPIRE"),
            addr=redis_cluster_addr or redis_addr,
            password=os.getenv("REDIS_PASSWORD", ""),
            model=RedisModel.CLUSTER if redis_cluster_addr else RedisModel.SINGLE,
        )
        if redis_cache.is_connected():
            logger.debug("Redis cache is connected")
            return redis_cache
        else:
            raise RuntimeError("Could not connect to Redis cache")



import os
import socket
import time
from typing import Optional

from pydantic import BaseModel


class SidInfo(BaseModel):
    sub: str
    location: str
    index: int
    local_ip: str
    local_port: str


sid_generator2: Optional["SidGenerator2"] = None


def init_sid(sid_info: SidInfo):
    global sid_generator2
    sid_generator2 = SidGenerator2(sid_info=sid_info)


class SidGenerator2:
    sid2 = 2

    def __init__(self, sid_info: SidInfo):

        self.sid_info = sid_info

        ip = socket.inet_aton(self.sid_info.local_ip)
        if ip:
            ip_sec3 = ip[2]
            ip_sec4 = ip[3]
            ip3 = ip_sec3 & 0xFF
            ip4 = ip_sec4 & 0xFF
            self.short_local_ip = f"{ip3:02x}{ip4:02x}"
        else:
            raise ValueError("Bad IP !! " + self.sid_info.local_ip)
        if len(self.sid_info.local_port) < 4:
            raise ValueError("Bad Port!! " + self.sid_info.local_port)

    def gen(self):
        pid = os.getpid() & 0xFF
        self.index = (self.sid_info.index + 1) & 0xFFFF
        tm_int = int(time.time() * 1000)
        tm = format(tm_int, "011x")
        sid = f"{self.sid_info.sub}{pid:04x}{self.sid_info.index:04x}@{self.sid_info.location}{tm[-11:]}{self.short_local_ip}{self.sid_info.local_port[:2]}{self.sid2}"
        return sid



import abc
from typing import Any, Type

from pydantic_settings import BaseSettings, PydanticBaseSettingsSource

from common.service.base import Service, ServiceType

# from common.settings.settings import ProjectSettings


class BaseSettingsService(Service):

    name = ServiceType.SETTINGS_SERVICE

    @property
    @abc.abstractmethod
    def setting_base(self) -> BaseSettings:
        raise NotImplementedError


class BaseRemoteSettings(PydanticBaseSettingsSource):

    def __init__(
        self,
        settings_cls: type[BaseSettings],
        dotenv_settings: PydanticBaseSettingsSource,
        cls: Type["BaseSettings"],
    ):
        super().__init__(settings_cls)
        self.dotenv_settings = dotenv_settings()

    @abc.abstractmethod
    def __call__(self) -> dict[str, Any]:
        raise NotImplementedError



"""
base.py
"""

import asyncio
from typing import TYPE_CHECKING, Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, Field

from common.audit_system.audit_api.base import ContentType, ContextList, Stage
from common.audit_system.enums import Status
from common.exceptions.errs import AuditServiceException

if TYPE_CHECKING:
    from common.otlp.trace.span import Span


class BaseFrameAudit(BaseModel):
    """
    基础帧审核对象，包含通用属性。
    """

    content: str

    status: Status = Status.STOP


class InputFrameAudit(BaseFrameAudit):
    """
    输入帧送审
    """

    content_type: ContentType = ContentType.TEXT
    context_list: list[ContextList] = Field(default_factory=list)


class OutputFrameAudit(BaseFrameAudit):
    """
    输出帧送审
    """

    frame_id: str = ""

    stage: Stage

    source_frame: Any

    not_need_submit: bool = False

    none_need_audit: bool = False

    model_config = ConfigDict(arbitrary_types_allowed=True)


class FrameAuditResult(BaseFrameAudit):
    """
    帧审核结果对象，包含审核状态和内容。
    """

    source_frame: Any = None

    error: Optional[AuditServiceException] = None

    model_config = ConfigDict(arbitrary_types_allowed=True)


class AuditContext(BaseModel):
    """
    审核上下文，维护当前会话状态，包括：
    - 所有帧拼接内容
    - 当前首句缓存
    - 是否因某一帧审核失败而停止帧审核
    """

    chat_sid: str

    template_id: str = ""

    chat_app_id: str = ""

    uid: str = ""

    output_queue: asyncio.Queue[Any] = Field(default_factory=asyncio.Queue)  # type: ignore[misc]

    error: Optional[AuditServiceException] = None

    pindex: int = 1

    first_sentence_audited: bool = False

    frame_blocked: bool = False

    remaining_content: str = ""

    all_content_frame_ids: List[str] = Field(default_factory=list)  # type: ignore[misc]
    all_source_frames: Dict[str, OutputFrameAudit] = Field(default_factory=dict)  # type: ignore[misc]

    audited_content: str = ""
    audited_content_frame_ids: List[str] = Field(default_factory=list)  # type: ignore[misc]

    frame_ids_on_screen: List[str] = Field(default_factory=list)  # type: ignore[misc]

    last_content_stage: Optional[Stage] = None

    model_config = ConfigDict(arbitrary_types_allowed=True)

    def add_source_content(self, output_frame: OutputFrameAudit):
        """
        添加原始帧内容到上下文中。
        :param output_frame: 输出帧送审对象
        """
        if output_frame.frame_id not in self.all_content_frame_ids:
            self.all_content_frame_ids.append(output_frame.frame_id)
            self.audited_content_frame_ids.append(output_frame.frame_id)
            self.all_source_frames[output_frame.frame_id] = output_frame

    async def add_audited_content(self, span: "Span"):
        """
        添加已审核完成的内容
        :param status: 当前状态
        :param span: Span对象，用于跟踪请求的上下文信息
        """
        is_all_consumer = True
        for idx, frame_id in enumerate(self.audited_content_frame_ids):
            output_frame_audit = self.all_source_frames[frame_id]
            if (
                output_frame_audit.content
                == self.audited_content[: len(output_frame_audit.content)]
            ):
                if output_frame_audit.frame_id not in self.frame_ids_on_screen:
                    frame_audit_result = FrameAuditResult(
                        content=output_frame_audit.content,
                        source_frame=output_frame_audit.source_frame,
                    )
                    await self.output_queue_put(frame_audit_result, span)
                self.audited_content = self.audited_content[
                    len(output_frame_audit.content) :
                ]
            else:
                self.audited_content_frame_ids = self.audited_content_frame_ids[idx:]
                is_all_consumer = False
                break
        if is_all_consumer:
            self.audited_content_frame_ids = []

    async def output_queue_put(
        self, frame_audit_result: FrameAuditResult, span: "Span"
    ):
        """
        将审核结果放入输出队列。
        :param frame_audit_result: 帧审核结果对象
        """
        event = f"当前可上屏内容: {frame_audit_result.content}"
        if frame_audit_result.error:
            event = f"{event}, 该上屏存在风险，风险信息: {frame_audit_result.error}"
        span.add_info_event(event)
        await self.output_queue.put(frame_audit_result)



import os
from typing import Optional

from common.service.base import ServiceFactory
from common.service.kafka.kafka_service import KafkaProducerService


class KafkaProducerServiceFactory(ServiceFactory):
    def __init__(self) -> None:
        super().__init__(KafkaProducerService)

    def create(self, servers: Optional[str] = None, **kwargs):
        """
        创建 KafkaProducerService 实例
        :param servers: Kafka bootstrap.servers
        :return: KafkaProducerService 实例
        """
        servers = servers or os.getenv("KAFKA_SERVERS")
        if not servers:
            raise ValueError("KAFKA_SERVERS 环境变量未配置")

        config = {"bootstrap.servers": servers, **kwargs}
        return KafkaProducerService(config)



from common.exceptions.base import BaseExc


class XingchenUtilsMASDKException(BaseExc):
    pass


class ErrorCode:
    Successes = 0
    MASDKClosedError = 0
    InitParamInvalidError = 9101
    AuthorizationCheckError = 9102
    AuthorizationCheckV2Error = 9103
    MetrologyCalcError = 9104
    ConcurrentAcquireConcError = 9105
    ConcurrentReleaseConcError = 9106
    CntInitFailedError = 9107
    ConcInitFailError = 9108
    MASDKUnknownError = 9999


Successes = XingchenUtilsMASDKException(0, "成功")
MASDKClosedError = XingchenUtilsMASDKException(0, "计量鉴权SDK功能关闭")
InitParamInvalidError = XingchenUtilsMASDKException(
    9101, "计量鉴权SDK初始化参数非法异常"
)
AuthorizationCheckError = XingchenUtilsMASDKException(
    9102, "计量鉴权SDK鉴权Check调用失败"
)
AuthorizationCheckV2Error = XingchenUtilsMASDKException(
    9103, "计量鉴权SDK鉴权CheckV2调用失败"
)
MetrologyCalcError = XingchenUtilsMASDKException(9104, "计量鉴权SDK计量调用失败")
ConcurrentAcquireConcError = XingchenUtilsMASDKException(
    9105, "计量鉴权SDK精准并发调用失败"
)
ConcurrentReleaseConcError = XingchenUtilsMASDKException(
    9106, "计量鉴权SDK精准并发释放失败"
)
CntInitFailedError = XingchenUtilsMASDKException(9107, "计量鉴权SDK计量鉴权初始化异常")
ConcInitFailError = XingchenUtilsMASDKException(9108, "计量鉴权SDK并发鉴权初始化异常")
MASDKUnknownError = XingchenUtilsMASDKException(9999, "未知异常")


class MASDKErrors:
    @classmethod
    def get_error(cls, code: int):
        for err in [
            Successes,
            MASDKClosedError,
            InitParamInvalidError,
            AuthorizationCheckError,
            AuthorizationCheckV2Error,
            MetrologyCalcError,
            ConcurrentAcquireConcError,
            ConcurrentReleaseConcError,
            CntInitFailedError,
            ConcInitFailError,
        ]:
            if err.c == code:
                return err
        return MASDKUnknownError



from contextlib import contextmanager

from loguru import logger
from sqlalchemy import Engine, create_engine
from sqlmodel import Session

from common.service.base import Service, ServiceType


class DatabaseService(Service):
    name = ServiceType.DATABASE_SERVICE

    def __init__(
        self,
        database_url: str,
        connect_timeout: int = 10,
        pool_size: int = 200,
        max_overflow: int = 800,
        pool_recycle: int = 3600,
    ):
        """
        :param database_url:    数据连接地址
        :param connect_timeout: 超时时间
        :param pool_size:       连接池大小
        :param max_overflow:    额外连接数
        :param pool_recycle:    重用连接之前的最大秒数，用于处理数据库服务器自动关闭长时间运行的连接的情况
        """
        self.database_url = database_url
        self.connect_timeout = connect_timeout
        self.pool_size = pool_size
        self.max_overflow = max_overflow
        self.pool_recycle = pool_recycle
        self.engine = self._create_engine()
        logger.debug("database init success")

    def _create_engine(self) -> "Engine":
        """Create the engine for the database."""

        return create_engine(
            self.database_url,
            echo=False,
            pool_size=self.pool_size,
            max_overflow=self.max_overflow,
            pool_recycle=self.pool_recycle,
        )

    def __enter__(self):
        self._session = Session(self.engine)
        return self._session

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is not None:  # If an exception has been raised
            logger.error(
                f"Session rollback because of exception: {exc_type.__name__} {exc_value}"
            )
            self._session.rollback()
        else:
            self._session.commit()
        self._session.close()

    def get_session(self):
        with Session(self.engine) as session:
            yield session


@contextmanager
def session_getter(db_service: "DatabaseService"):
    try:
        session = Session(db_service.engine)
        yield session
    except Exception as e:
        # print("Session rollback because of exception:", e)
        logger.debug(f"Session rollback because of exception: {e}")
        session.rollback()
        raise
    finally:
        session.close()



"""
calc.py
"""

import ctypes

from common.metrology_auth.base import BaseClass


class Metrology(BaseClass):

    def __init__(self, ctype_filename):
        self.lib = self.get_lib(ctype_filename)

        class CalcReturnType(ctypes.Structure):
            _fields_ = [("r0", ctypes.c_int), ("r1", ctypes.c_char_p)]

        self.CalcReturnType = CalcReturnType

        self.lib.Calc.restype = self.CalcReturnType

        self.lib.Calc.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_longlong,
        ]

        self.lib.Calc_Init.restype = ctypes.c_char_p
        self.lib.Calc_Init.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_int,
            ctypes.c_char_p,
        ]

        self.lib.Calc_Fini.restype = None
        self.lib.Calc_Fini.argtypes = []

        self.calc_inited = False

    def calc_init(self, url, pro, gro, service, version, mode, sname):
        url = url.encode()
        pro = pro.encode()
        gro = gro.encode()
        service = service.encode()
        version = version.encode()
        sname = sname.encode()
        result = self.lib.Calc_Init(url, pro, gro, service, version, mode, sname)
        if not result:
            self.calc_inited = True
            return None
        else:
            return result.decode()

    def calc(self, appid, channel, funcs, c):
        appid = appid.encode()
        channel = channel.encode()
        funcs = funcs.encode()
        calc_result = self.lib.Calc(appid, channel, funcs, c)
        return calc_result.r0, calc_result.r1.decode() if calc_result.r1 else None

    def calc_fini(self):
        self.lib.Calc_Fini()



import base64
import hashlib
import hmac
from datetime import datetime
from time import mktime
from urllib import parse
from urllib.parse import urlencode, urlparse
from wsgiref.handlers import format_date_time


class HMACAuth:

    @staticmethod
    def build_auth_request_url(request_url, method="GET", api_key="", api_secret=""):
        values = HMACAuth.build_auth_params(request_url, method, api_key, api_secret)
        return request_url + "?" + urlencode(values)

    @staticmethod
    def build_auth_params(request_url, method="GET", api_key="", api_secret="") -> dict:
        url_result = parse.urlparse(request_url)
        date = format_date_time(mktime(datetime.now().timetuple()))
        signature_origin = "host: {}\ndate: {}\n{} {} HTTP/1.1".format(
            url_result.hostname, date, method, url_result.path
        )
        signature_sha_bytes = hmac.new(
            api_secret.encode("utf-8"),
            signature_origin.encode("utf-8"),
            digestmod=hashlib.sha256,
        ).digest()
        signature_sha_str = base64.b64encode(signature_sha_bytes).decode(
            encoding="utf-8"
        )
        authorization_origin = (
            'api_key="%s", algorithm="%s", headers="%s", signature="%s"'
            % (api_key, "hmac-sha256", "host date request-line", signature_sha_str)
        )
        authorization = base64.b64encode(authorization_origin.encode("utf-8")).decode(
            encoding="utf-8"
        )
        values = {
            "host": url_result.hostname,
            "date": date,
            "authorization": authorization,
        }
        return values

    @staticmethod
    def build_auth_header(request_url, method="GET", api_key="", api_secret="") -> dict:
        u = urlparse(request_url)
        host = u.hostname
        path = u.path
        now = datetime.now()
        date = format_date_time(mktime(now.timetuple()))
        m = hashlib.sha256(bytes("".encode(encoding="utf-8"))).digest()
        digest_bytes = "SHA256=" + base64.b64encode(m).decode(encoding="utf-8")
        signatureStr = "host: " + str(host) + "\n"
        signatureStr += "date: " + date + "\n"
        signatureStr += method + " " + path + " " + "HTTP/1.1" + "\n"
        # signatureStr += self.HttpMethod + " " + " " + self.HttpProto + "\n"
        signatureStr += "digest: " + digest_bytes
        # print("signatureStr:%s  **********" % signatureStr)
        signature = hmac.new(
            bytes(api_secret, encoding="UTF-8"),
            bytes(signatureStr, encoding="UTF-8"),
            digestmod=hashlib.sha256,
        ).digest()
        sign = base64.b64encode(signature).decode(encoding="utf-8")
        # print("result:%s    ************" % result)

        authHeader = (
            'api_key="%s", algorithm="%s", '
            'headers="host date request-line digest", '
            'signature="%s"' % (api_key, "hmac-sha256", sign)
        )
        # print(authHeader)
        headers = {
            "Method": method,
            "Host": host,
            "Date": date,
            "Digest": digest_bytes,
            "Authorization": authHeader,
        }
        return headers



import os

from confluent_kafka import Producer
from loguru import logger

from common.service.base import Service, ServiceType


class KafkaProducerService(Service):
    name = ServiceType.KAFKA_PRODUCER_SERVICE

    def __init__(self, config: dict):
        """
        Kafka 生产者服务封装
        :param config: Kafka 配置
        """
        self.config = config
        self.producer = Producer(**config)

    def send(
        self,
        topic: str,
        value: str,
        callback=None,
        timeout: int = int(os.getenv("KAFKA_TIMEOUT", 10)),
    ):
        """
        发送 Kafka 消息
        :param topic: Kafka topic
        :param value: 消息内容（已序列化的 JSON 字符串）
        :param callback: 回调函数
        :param timeout: poll timeout (秒)
        """
        if not callback:
            callback = self._delivery_report
        try:
            self.producer.produce(topic=topic, value=value, callback=callback)
            self.producer.poll(timeout)
        except Exception as e:
            logger.error(f"Kafka message send failed: {e}")
            raise e

    def _delivery_report(self, err, msg):
        """
        消息发送回调函数
        :param err:
        :param msg:
        :return:
        """
        if err is not None:
            logger.error("Message delivery failed: {}".format(err))
        else:
            logger.info(
                "Message delivered to {} [{}]".format(msg.topic(), msg.partition())
            )



from typing import ClassVar

from pydantic import Field

from common.otlp.log_trace.node_log import NodeLog
from common.otlp.log_trace.node_trace_log import NodeTraceLog


class WorkflowLog(NodeTraceLog):

    workflow_stream_node_types: ClassVar[list] = Field(default=["message", "node-end"])

    def add_node_log(self, node_logs: list[NodeLog]) -> None:
        if not node_logs:
            return
        """
        遍历节点，获取首响时间
        规则：
            如果遍历到的第一个消息节点，设置首响时间为 (消息节点的开始时间 - 开始时间)
            如果遍历到第一个是
        """
        if self.first_frame_duration == -1:

            for i, node_log in enumerate(node_logs):
                node_type = node_log.node_id.split(":")[0]
                if node_type in self.workflow_stream_node_types:
                    self.first_frame_duration = node_log.start_time - self.start_time
                    break

        self.trace.extend(node_logs)



import json
import time
import uuid
from typing import Dict, List, Optional

from pydantic import BaseModel, Field

from common.otlp.log_trace.base import Usage
from common.otlp.log_trace.node_log import NodeLog
from common.service.oss.base_oss import BaseOSSService


class Status(BaseModel):
    code: int = 0
    message: str = ""


class NodeTraceLog(BaseModel):
    service_id: str
    sid: str
    app_id: str = Field(default="", description="应用ID")
    uid: str = Field(default="", description="用户ID")
    chat_id: str = Field(default="", description="会话ID")
    sub: str
    caller: str = ""
    log_caller: str = ""

    question: str = Field(default="", description="问题")
    answer: str = Field(default="", description="答案")

    start_time: int = Field(default_factory=lambda: int(time.time() * 1000))
    end_time: int = Field(default_factory=lambda: int(time.time() * 1000))
    duration: int = 0
    first_frame_duration: float = -1.0

    srv: Dict[str, str] = {}
    srv_tag: Dict[str, str] = {}
    status: Status = Status()
    usage: Usage = Usage()
    version: str = "v2.0.0"
    trace: List[NodeLog] = Field(default_factory=list)

    class Config:
        arbitrary_types_allowed = True

    def add_q(self, question: str):
        """
        description: add q
        """
        self.question = question

    def add_a(self, answer: str):
        """
        description: add a
        """
        self.answer = answer

    def add_first_frame_duration(self, first_frame_duration: int):
        """
        description: add first frame duration
        """
        self.first_frame_duration = first_frame_duration

    def add_srv(self, key: str, value: str):
        self.srv[key] = value
        self.srv_tag[key] = value

    def set_end(self):
        """
        日志结束
        :return:
        """
        self.end_time = int(time.time() * 1000)
        self.duration = self.end_time - self.start_time
        for i, node_log in enumerate(self.trace):
            self.usage.total_tokens += node_log.data.usage.total_tokens
            self.usage.prompt_tokens += node_log.data.usage.prompt_tokens
            self.usage.question_tokens += node_log.data.usage.question_tokens
            self.usage.completion_tokens += node_log.data.usage.completion_tokens

    def set_status(self, code: int, message: str):
        self.status.code = code
        self.status.message = message

    def add_node_log(self, node_logs: list[NodeLog]):
        if not node_logs:
            return
        self.trace.extend(node_logs)

    def add_func_log(self, node_logs: list[NodeLog]):
        self.add_node_log(node_logs)

    def to_json(self, large_field_save_service: Optional[BaseOSSService] = None) -> str:
        """
        返回JSON字符串。超过5kb的value值，存储在对应存储服务中
        :return:
        """

        import sys

        def is_large_string(s: str, limit=5 * 1024) -> bool:
            return isinstance(s, str) and sys.getsizeof(s.encode("utf-8")) > limit

        def process_data(data):
            if isinstance(data, dict):
                return {k: process_data(v) for k, v in data.items()}
            elif isinstance(data, list):
                return [process_data(item) for item in data]
            elif isinstance(data, str):
                if is_large_string(data):
                    if isinstance(large_field_save_service, BaseOSSService):
                        return large_field_save_service.upload_file(
                            f"{uuid.uuid4().hex}.txt", data.encode("utf-8")
                        )
                    return data
                else:
                    return data
            else:
                return data

        result = process_data(self.model_dump())

        def json_fallback(obj):
            if isinstance(obj, set):
                return list(obj)

        return json.dumps(result, ensure_ascii=False, default=json_fallback)



import os

from common.otlp.args import global_otlp_trace_args
from common.otlp.trace.trace import init_trace
from common.service.base import ServiceFactory, ServiceType
from common.service.otlp.span.span_service import OtlpSpanService


def init_otlp_span():

    # global global_otlp_trace_args

    if global_otlp_trace_args.inited:
        return

    global_otlp_trace_args.otlp_endpoint = os.getenv("OTLP_ENDPOINT", "")
    global_otlp_trace_args.otlp_service_name = os.getenv("SERVICE_NAME", "")
    global_otlp_trace_args.otlp_dc = os.getenv("SERVICE_LOCATION", "")

    global_otlp_trace_args.trace_timeout = int(os.getenv("OTLP_TRACE_TIMEOUT", "3000"))
    global_otlp_trace_args.trace_max_queue_size = int(
        os.getenv("OTLP_TRACE_MAX_QUEUE_SIZE", "2048")
    )
    global_otlp_trace_args.trace_schedule_delay_millis = int(
        os.getenv("OTLP_TRACE_SCHEDULE_DELAY_MILLIS", "3000")
    )
    global_otlp_trace_args.trace_max_export_batch_size = int(
        os.getenv("OTLP_TRACE_MAX_EXPORT_BATCH_SIZE", "512")
    )
    global_otlp_trace_args.trace_export_timeout_millis = int(
        os.getenv("OTLP_TRACE_EXPORT_TIMEOUT_MILLIS", "3000")
    )
    init_trace(
        endpoint=global_otlp_trace_args.otlp_endpoint,
        service_name=global_otlp_trace_args.otlp_service_name,
        timeout=global_otlp_trace_args.trace_timeout,
        max_queue_size=global_otlp_trace_args.trace_max_queue_size,
        schedule_delay_millis=global_otlp_trace_args.trace_schedule_delay_millis,
        max_export_batch_size=global_otlp_trace_args.trace_max_export_batch_size,
        export_timeout_millis=global_otlp_trace_args.trace_export_timeout_millis,
    )


class OtlpSpanFactory(ServiceFactory):
    name = ServiceType.OTLP_SPAN_SERVICE

    def __init__(self):
        super().__init__(OtlpSpanService)

    def create(self):
        init_otlp_span()
        return OtlpSpanService()



from common.exceptions.base import BaseExc


class BaseCommonException(BaseExc):
    """基础common异常类"""

    pass


class OssServiceException(BaseCommonException):
    """oss服务异常"""

    pass


class AuditServiceException(BaseCommonException):
    """audit服务异常"""

    pass



"""
utils.py
"""

import re
from typing import Tuple

END_SYMBOLS = [
    "\n",
    "\n\n",
    "。",
    ".",
    "！",
    "!",
    "？",
    "?",
    "；",
    ";",
]

NON_END_SYMBOLS = [
    "，",
    ",",
    "：",
    ":",
    "、",
    "》",
    ")",
    "）",
    "】",
    "]",
    "}",
    "……",
]

END_SYMBOLS_PATTERN = re.compile(
    rf'.+?(?:{"|".join(map(re.escape, END_SYMBOLS))})', re.DOTALL
)

NON_END_SYMBOL_SET = set(NON_END_SYMBOLS)

MAX_SENTENCE_LEN = 50
ALL_SENTENCE_LEN = -1


class Sentence:
    """
    处理文本内容的句子
    """

    @staticmethod
    def find_valid_sentence(
        content: str, fallback_length: int = MAX_SENTENCE_LEN
    ) -> tuple[list[str], str]:
        """
        查找文本中的首个有效句子，优先返回完整句子，如果没有完整句子则用非结束性标点分段，否则返回前50个字符
        :param content:
        :param fallback_length:
        :return: (首句, 剩余文本)
        """

        sentences: list = []
        remaining = ""

        if not content:
            return sentences, ""

        if fallback_length == ALL_SENTENCE_LEN:
            return [content], ""

        sentences, remaining = Sentence._extract_first_end_symbol(
            content, fallback_length
        )
        if sentences:
            return sentences, remaining

        sentence, remaining = Sentence._extract_before_last_non_end_symbol(
            content, fallback_length
        )
        if sentence:
            sentences.append(sentence)
            return sentences, remaining

        first_sentence = (
            content[:fallback_length]
            if fallback_length != ALL_SENTENCE_LEN
            else content
        )
        remaining = (
            content[fallback_length:] if fallback_length != ALL_SENTENCE_LEN else ""
        )
        sentences.append(first_sentence)
        return sentences, remaining

    @staticmethod
    def _extract_first_end_symbol(
        text: str, fallback_length: int = MAX_SENTENCE_LEN
    ) -> Tuple[list, str]:
        """
        提取文本中的句子，找到所有非结束性标点符号的句子
        :param text:
        :return:
        """

        sentences_temp = END_SYMBOLS_PATTERN.findall(text)
        sentences = []
        if fallback_length != ALL_SENTENCE_LEN:
            one = ""
            for _, s in enumerate(sentences_temp):
                if len(one) + len(s) <= fallback_length:
                    one += s
                else:
                    if one:
                        sentences.append(one)
                    one = s
            if one:
                sentences.append(one)

        matched_len = sum(len(s) for s in sentences)
        remainder = text[matched_len:]
        return sentences, remainder

    @staticmethod
    def _extract_before_last_non_end_symbol(
        text: str, fallback_length: int = MAX_SENTENCE_LEN
    ) -> Tuple[str, str]:
        """
        提取文本中最后一个非结束性符号之前的内容和剩余文本
        :param text:
        :return: 包含该符号的前缀 + 剩余文本
        """
        max_len = max(len(sym) for sym in NON_END_SYMBOLS)
        need_deal_text = (
            text[:fallback_length] if fallback_length != ALL_SENTENCE_LEN else text
        )
        for i in range(len(need_deal_text) - 1, -1, -1):
            for ll in range(1, max_len + 1):
                start = i - ll + 1
                if start < 0:
                    continue
                symbol = need_deal_text[start : i + 1]
                if symbol in NON_END_SYMBOL_SET:
                    sentence = need_deal_text[: i + 1]
                    remaining = text[len(sentence) :]
                    return sentence, remaining

        return "", ""

    @staticmethod
    def has_end_symbol(text: str) -> bool:
        """
        检查文本是否包含结束性标点符号
        :param text:
        :return:
        """
        return any(sym in text for sym in END_SYMBOLS)

    @staticmethod
    def split_and_keep_delimiters(text: str, separators: list[str]) -> list[str]:
        """
        拆分文本，保留分隔符
        :param text:
        :param separators:
        :return:
        """
        result = []
        current = ""

        for char in text:
            if char in separators:
                result.append(current + char)
                current = ""
            else:
                current += char

        if current:
            result.append(current)

        return result



"""
__init__.py
"""

import json
import os
import platform
import socket
from contextlib import contextmanager
from dataclasses import dataclass
from datetime import datetime
from typing import Optional

import toml

from common.metrology_auth.calc import Metrology
from common.metrology_auth.conc import Concurrent
from common.metrology_auth.errors import ErrorCode, MASDKErrors
from common.metrology_auth.licc import Authorization


@dataclass
class MASDKRequest:
    sid: str
    appid: str
    channel: str
    function: str
    cnt: int
    uid: Optional[str] = None

    def to_dict(self):
        return {
            "sid": self.sid,
            "appid": self.appid,
            "channel": self.channel,
            "function": self.function,
            "cnt": self.cnt,
            "uid": self.uid,
        }


@dataclass
class MASDKResponse:
    code: int
    msg: str
    log: str


class MetrologyAuthorization:
    def __init__(
        self, url, pro, gro, service, version, mode, sname, channel_list, ctype_filename
    ):
        self.metrology = Metrology(ctype_filename)
        self.authorization = Authorization(ctype_filename)
        self.url = url
        self.pro = pro
        self.gro = gro
        self.service = service
        self.version = version
        self.mode = mode
        self.sname = sname
        self.channel_list = channel_list

        _ = self.authorization.check_init(
            self.url,
            self.pro,
            self.gro,
            self.service,
            self.version,
            self.mode,
            self.channel_list,
            self.sname,
        )
        # print(f"Check_Init result: {check_init_result}")

        _ = self.metrology.calc_init(
            self.url,
            self.pro,
            self.gro,
            self.service,
            self.version,
            self.mode,
            self.sname,
        )
        # print(f"Calc_Init result: {result}")

    def excute(self, masdk_request: MASDKRequest):
        now = datetime.now()
        tag = now.strftime("%Y-%m-%d %H:%M:%S.%f")
        r0, r1, r2 = self.authorization.check(
            masdk_request.appid,
            masdk_request.uid,
            masdk_request.channel,
            [masdk_request.function],
            tag,
        )

        if r0 != "null":
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": {
                    "r0": r0,
                    "r1": r1,
                    "r2": r2,
                },
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.AuthorizationCheckError).c,
                msg=MASDKErrors.get_error(ErrorCode.AuthorizationCheckError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )

        r0, r1 = self.metrology.calc(
            masdk_request.appid,
            masdk_request.channel,
            masdk_request.function,
            masdk_request.cnt,
        )

        return MASDKResponse(
            code=MASDKErrors.get_error(ErrorCode.Successes).c,
            msg=MASDKErrors.get_error(ErrorCode.Successes).m,
            log="",
        )


class ConcurrentAuthorization:
    def __init__(
        self, url, pro, gro, service, version, mode, sname, channel_list, ctype_filename
    ):
        self.concurrent = Concurrent(ctype_filename)
        self.authorization = Authorization(ctype_filename)
        self.url = url
        self.pro = pro
        self.gro = gro
        self.service = service
        self.version = version
        self.mode = mode
        self.sname = sname
        self.channel_list = channel_list

        _ = self.authorization.check_init(
            self.url,
            self.pro,
            self.gro,
            self.service,
            self.version,
            self.mode,
            self.channel_list,
            self.sname,
        )

        _ = self.concurrent.conc_init(
            self.url,
            self.pro,
            self.gro,
            self.service,
            self.version,
            self.mode,
            get_local_ip(),
            self.sname,
        )

    def acquire(self, masdk_request: MASDKRequest):
        now = datetime.now()
        tag = now.strftime("%Y-%m-%d %H:%M:%S.%f")
        r0, r1, r2 = self.authorization.checkV2(
            masdk_request.appid,
            masdk_request.uid,
            masdk_request.channel,
            [masdk_request.function],
            tag,
        )

        if r0 != "null":
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": {
                    "r0": r0,
                    "r1": r1,
                    "r2": r2,
                },
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.AuthorizationCheckV2Error).c,
                msg=MASDKErrors.get_error(ErrorCode.AuthorizationCheckV2Error).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        for i in range(masdk_request.cnt):
            r0, r1, r2, r3 = self.concurrent.acquire_conc(
                masdk_request.sid,
                masdk_request.appid,
                masdk_request.channel,
                masdk_request.function,
            )

            if r0 != 1:
                logInfo = {
                    "request": masdk_request.to_dict(),
                    "response": {
                        "r0": r0,
                        "r1": r1,
                        "r2": r2,
                        "r3": r3,
                    },
                }
                return MASDKResponse(
                    code=MASDKErrors.get_error(ErrorCode.ConcurrentAcquireConcError).c,
                    msg=MASDKErrors.get_error(ErrorCode.ConcurrentAcquireConcError).m,
                    log=json.dumps(logInfo, ensure_ascii=False),
                )
        return MASDKResponse(
            code=MASDKErrors.get_error(ErrorCode.Successes).c,
            msg=MASDKErrors.get_error(ErrorCode.Successes).m,
            log="",
        )

    def release(self, masdk_request: MASDKRequest):
        for i in range(masdk_request.cnt):
            result = self.concurrent.release_conc(
                masdk_request.sid,
                masdk_request.appid,
                masdk_request.channel,
                masdk_request.function,
            )
            if result != "":
                logInfo = {
                    "request": masdk_request.to_dict(),
                    "response": {
                        "result": result,
                    },
                }
                return MASDKResponse(
                    code=MASDKErrors.get_error(ErrorCode.ConcurrentReleaseConcError).c,
                    msg=MASDKErrors.get_error(ErrorCode.ConcurrentReleaseConcError).m,
                    log=json.dumps(logInfo, ensure_ascii=False),
                )
        return MASDKResponse(
            code=MASDKErrors.get_error(ErrorCode.Successes).c,
            msg=MASDKErrors.get_error(ErrorCode.Successes).m,
            log="",
        )

    @contextmanager
    def excute(self, masdk_request: MASDKRequest):
        if not os.getenv("MASDK_SWITCH", 0):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": f"MASDK_SWITCH is {os.getenv('MASDK_SWITCH')}",
            }
            self.masdk_response = MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.MASDKClosedError).c,
                msg=MASDKErrors.get_error(ErrorCode.MASDKClosedError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
            yield self
            return
        self.masdk_response = MASDKResponse(
            code=MASDKErrors.get_error(ErrorCode.Successes).c,
            msg=MASDKErrors.get_error(ErrorCode.Successes).m,
            log="",
        )
        now = datetime.now()
        tag = now.strftime("%Y-%m-%d %H:%M:%S.%f")
        r0, r1, r2 = self.authorization.checkV2(
            masdk_request.appid,
            masdk_request.uid,
            masdk_request.channel,
            [masdk_request.function],
            tag,
        )
        if r0 != "null":
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": {
                    "r0": r0,
                    "r1": r1,
                    "r2": r2,
                },
            }
            self.masdk_response = MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.AuthorizationCheckV2Error).c,
                msg=MASDKErrors.get_error(ErrorCode.AuthorizationCheckV2Error).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        else:
            for i in range(masdk_request.cnt):
                r0, r1, r2, r3 = self.concurrent.acquire_conc(
                    masdk_request.sid,
                    masdk_request.appid,
                    masdk_request.channel,
                    masdk_request.function,
                )
                if r0 != 1:
                    logInfo = {
                        "request": masdk_request.to_dict(),
                        "response": {
                            "r0": r0,
                            "r1": r1,
                            "r2": r2,
                            "r3": r3,
                        },
                    }
                    self.masdk_response = MASDKResponse(
                        code=MASDKErrors.get_error(
                            ErrorCode.ConcurrentAcquireConcError
                        ).c,
                        msg=MASDKErrors.get_error(
                            ErrorCode.ConcurrentAcquireConcError
                        ).m,
                        log=json.dumps(logInfo, ensure_ascii=False),
                    )
        try:
            yield self
        finally:
            for i in range(masdk_request.cnt):
                result = self.concurrent.release_conc(
                    masdk_request.sid,
                    masdk_request.appid,
                    masdk_request.channel,
                    masdk_request.function,
                )
                if result != "":
                    logInfo = {
                        "request": masdk_request.to_dict(),
                        "response": {
                            "result": result,
                        },
                    }
                    self.masdk_response = MASDKResponse(
                        code=MASDKErrors.get_error(
                            ErrorCode.ConcurrentReleaseConcError
                        ).c,
                        msg=MASDKErrors.get_error(
                            ErrorCode.ConcurrentReleaseConcError
                        ).m,
                        log=json.dumps(logInfo, ensure_ascii=False),
                    )


def copy_toml(source_file, target_file):
    try:
        with open(source_file, "r", encoding="utf-8") as f:
            data = toml.load(f)
        with open(target_file, "w", encoding="utf-8") as f:
            toml.dump(data, f)
    except Exception:
        return MASDKErrors.get_error(ErrorCode.InitParamInvalidError)
    return


def get_local_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        ip_address = s.getsockname()[0]
    finally:
        s.close()
    return ip_address


class MASDK:

    def get_ctype_file_for_platform(self):
        current_file_path = os.path.abspath(__file__)
        operate_system = platform.system()
        if operate_system == "Windows":
            ctype_filename = os.path.dirname(current_file_path) + "/ma_sdk_windows.so"
        elif operate_system == "Linux":
            ctype_filename = os.path.dirname(current_file_path) + "/ma_sdk_linux_x64.so"
        elif operate_system == "Darwin":
            ctype_filename = (
                os.path.dirname(current_file_path) + "/ma_sdk_macos_arm64.so"
            )
        else:
            raise (MASDKErrors.get_error(ErrorCode.InitParamInvalidError))
        return ctype_filename

    def get_polaris_flag(self):
        polaris_flag = False
        if (
            self._polaris_url != ""
            and self._polaris_project != ""
            and self._polaris_group != ""
            and self._polaris_service != ""
            and self._polaris_version != ""
        ):
            polaris_flag = True
        return polaris_flag

    def get_mode(self):
        if self._rpc_config_file is not None:
            err = copy_toml(self._rpc_config_file, "./ma-sdk.toml")
            if err:
                raise (err)
            else:
                mode = 0
        elif self._polaris_flag is True:
            mode = 1
        else:
            err = copy_toml(
                os.path.dirname(self._current_file_path) + "/ma-sdk-default.toml",
                "./ma-sdk.toml",
            )
            if err:
                raise (err)
            else:
                mode = 0
        return mode

    def __init__(
        self,
        channel_list,
        strategy_type,
        polaris_url="",
        polaris_project="",
        polaris_group="",
        polaris_service="",
        polaris_version="",
        rpc_config_file=None,
        metrics_service_name=None,
    ):
        self._current_file_path = os.path.abspath(__file__)
        self._channel_list = channel_list
        self._strategy_type = strategy_type
        self._polaris_url = polaris_url
        self._polaris_project = polaris_project
        self._polaris_group = polaris_group
        self._polaris_service = polaris_service
        self._polaris_version = polaris_version
        self._rpc_config_file = rpc_config_file
        self._metrics_service_name = metrics_service_name

        self._ctype_filename = self.get_ctype_file_for_platform()

        self._polaris_flag = self.get_polaris_flag()
        self._mode = self.get_mode()

        for item in strategy_type:
            if item == "cnt":
                self.create_modular_method()
            if item == "conc":
                self.create_concurrent_method()

    def create_modular_method(self):
        self.modular_method = MetrologyAuthorization(
            self._polaris_url,
            self._polaris_project,
            self._polaris_group,
            self._polaris_service,
            self._polaris_version,
            self._mode,
            self._metrics_service_name,
            self._channel_list,
            self._ctype_filename,
        )

    def create_concurrent_method(self):
        self.concurrent_method = ConcurrentAuthorization(
            self._polaris_url,
            self._polaris_project,
            self._polaris_group,
            self._polaris_service,
            self._polaris_version,
            self._mode,
            self._metrics_service_name,
            self._channel_list,
            self._ctype_filename,
        )

    def metrology_authorization(self, masdk_request: MASDKRequest):
        if not os.getenv("MASDK_SWITCH", 0):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": f"MASDK_SWITCH is {os.getenv('MASDK_SWITCH')}",
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.MASDKClosedError).c,
                msg=MASDKErrors.get_error(ErrorCode.MASDKClosedError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        if not hasattr(self, "modular_method"):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": "MASDK has no modular_method",
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.CntInitFailedError).c,
                msg=MASDKErrors.get_error(ErrorCode.CntInitFailedError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        return self.modular_method.excute(masdk_request)

    def concurrent_authorization(self, masdk_request: MASDKRequest):
        return self.concurrent_method.excute(masdk_request)

    def acquire_concurrent(self, masdk_request: MASDKRequest):
        if not os.getenv("MASDK_SWITCH", 0):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": f"MASDK_SWITCH is {os.getenv('MASDK_SWITCH')}",
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.MASDKClosedError).c,
                msg=MASDKErrors.get_error(ErrorCode.MASDKClosedError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        if not hasattr(self, "concurrent_method"):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": "MASDK has no concurrent_method",
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.ConcInitFailError).c,
                msg=MASDKErrors.get_error(ErrorCode.ConcInitFailError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        return self.concurrent_method.acquire(masdk_request)

    def release_concurrent(self, masdk_request: MASDKRequest):
        if not os.getenv("MASDK_SWITCH", 0):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": f"MASDK_SWITCH is {os.getenv('MASDK_SWITCH')}",
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.MASDKClosedError).c,
                msg=MASDKErrors.get_error(ErrorCode.MASDKClosedError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        if not hasattr(self, "concurrent_method"):
            logInfo = {
                "request": masdk_request.to_dict(),
                "response": "MASDK has no concurrent_method",
            }
            return MASDKResponse(
                code=MASDKErrors.get_error(ErrorCode.ConcInitFailError).c,
                msg=MASDKErrors.get_error(ErrorCode.ConcInitFailError).m,
                log=json.dumps(logInfo, ensure_ascii=False),
            )
        return self.concurrent_method.release(masdk_request)



import json
import time
import uuid
from typing import Any, Dict, Set

from pydantic import BaseModel, Field

from common.otlp.log_trace.base import Usage


class Data(BaseModel):
    input: Dict[str, Any] = {}
    output: Dict[str, Any] = {}
    config: Dict[str, Any] = {}
    usage: Usage = Usage()


class NodeLog(BaseModel):
    id: str = Field(default_factory=lambda: uuid.uuid4().hex)
    sid: str = ""

    node_id: str = ""
    node_type: str = ""
    node_name: str = ""

    func_id: str = ""
    func_type: str = ""
    func_name: str = ""

    next_log_ids: Set[str] = set()

    start_time: int = Field(default_factory=lambda: int(time.time() * 1000))
    end_time: int = Field(default_factory=lambda: int(time.time() * 1000))
    duration: int | tuple = (0,)
    first_frame_duration: int = -1
    node_first_cost_time: float = -1

    llm_output: str = ""
    running_status: bool = True
    data: Data = Data()
    logs: list[str] = []

    def __init__(
        self,
        sid: str,
        func_id: str = "",
        func_name: str = "",
        func_type: str = "",
        **kwargs,
    ):
        node_id = kwargs.get("node_id", "")
        node_type = kwargs.get("node_type", "")
        node_name = kwargs.get("node_name", "")

        func_id = func_id if func_id else node_id
        func_name = func_name if func_name else node_name
        func_type = func_type if func_type else node_type

        super().__init__(
            sid=sid, func_id=func_id, func_name=func_name, func_type=func_type, **kwargs
        )

    def set_next_node_id(self, next_id):
        """
        设置下一个节点ID
        :param next_id:
        :return:
        """
        self.next_log_ids.add(next_id)

    def set_first_frame_duration(self):
        """
        设置首帧时间
        :return:
        """
        self.first_frame_duration = int(time.time() * 1000) - self.start_time

    def set_node_first_cost_time(self, cost_time):
        self.node_first_cost_time = cost_time

    def set_start(self):
        self.start_time = int(time.time() * 1000)

    def set_end(self):
        """
        日志结束
        :return:
        """
        self.end_time = int(time.time() * 1000)
        self.duration = self.end_time - self.start_time

    def append_input_data(self, key: str, data: Any):
        """
        添加节点输入数据
        :param key:
        :param data:
        :return:
        """
        self.data.input.update({key: data})

    def append_output_data(self, key: str, data: Any):
        """
        添加节点输出数据
        :param key:
        :param data:
        :return:
        """
        self.data.output.update({key: data})

    def append_usage_data(self, data: Any):
        """
        添加大模型用户数据
        :param key:
        :param data:
        :return:
        """
        self.data.usage.total_tokens = data.get("total_tokens", 0)
        self.data.usage.question_tokens = data.get("question_tokens", 0)
        self.data.usage.prompt_tokens = data.get("prompt_tokens", 0)
        self.data.usage.completion_tokens = data.get("completion_tokens", 0)

    def append_config_data(self, data: Dict[str, Any]):
        """
        添加配置数据，主要于节点参数
        :param data:
        :return:
        """
        self.data.config.update(data)

    def _add_log(self, log_level: str, content: str):
        """
        添加日志
        :param log:
        :return:
        """
        log = {
            "level": log_level,
            "message": content,
            "time": time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()),
        }
        self.logs.append(json.dumps(log, ensure_ascii=False))

    def add_info_log(self, log: str):
        """
        添加信息日志
        :param log:
        :return:
        """
        self._add_log("INFO", log)

    def add_error_log(self, log: str):
        """
        添加信息日志
        :param log:
        :return:
        """
        self._add_log("ERROR", log)



import ctypes


class BaseClass:
    """
    Base class for all classes in this module
    """

    _lib = None

    @classmethod
    def get_lib(cls, ctype_filename: str) -> ctypes.CDLL:
        """
        Get the library
        """
        if BaseClass._lib is None:
            # print("Loading .so file for the first time...")
            BaseClass._lib = ctypes.CDLL(ctype_filename)
        return BaseClass._lib



import os
from typing import Optional

from common.service.base import ServiceFactory
from common.service.db.db_service import DatabaseService


class DatabaseServiceFactory(ServiceFactory):
    def __init__(self):
        super().__init__(DatabaseService)

    def create(self):
        """
        Create a new DatabaseService instance.
        :param database_url:
        :return:
        """
        host = os.getenv("MYSQL_HOST")
        port = os.getenv("MYSQL_PORT")
        user = os.getenv("MYSQL_USER")
        password = os.getenv("MYSQL_PASSWORD")
        db = os.getenv("MYSQL_DB")
        database_url = f"mysql+pymysql://{user}:{password}@{host}:{port}/{db}"
        return DatabaseService(database_url=database_url)



import os

from common.otlp.args import global_otlp_sid_args
from common.otlp.sid import SidInfo, init_sid
from common.service.base import ServiceFactory, ServiceType
from common.service.otlp.sid.sid_service import OtlpSidService


def init_otlp_sid():

    # global global_otlp_sid_args

    if global_otlp_sid_args.inited:
        return

    global_otlp_sid_args.otlp_endpoint = os.getenv("OTLP_ENDPOINT", "")
    global_otlp_sid_args.otlp_service_name = os.getenv("SERVICE_NAME", "")
    global_otlp_sid_args.otlp_dc = os.getenv("SERVICE_LOCATION", "")

    global_otlp_sid_args.sid_sub = os.getenv("SERVICE_SUB", "svc")
    global_otlp_sid_args.sid_location = os.getenv("SERVICE_LOCATION", "src")
    global_otlp_sid_args.sid_local_port = os.getenv("SERVICE_PORT")
    init_sid(
        SidInfo(
            sub=global_otlp_sid_args.sid_sub,
            location=global_otlp_sid_args.sid_location,
            local_ip=global_otlp_sid_args.sid_ip,
            local_port=global_otlp_sid_args.sid_local_port,
            index=0,
        )
    )


class OtlpSidFactory(ServiceFactory):
    name = ServiceType.OTLP_SID_SERVICE

    def __init__(self):
        super().__init__(OtlpSidService)

    def create(self):
        init_otlp_sid()
        return OtlpSidService()



from common.service.base import ServiceFactory, ServiceType
from common.service.otlp.node_log.node_log_service import OtlpNodeLogService


class OtlpNodeLogFactory(ServiceFactory):
    name = ServiceType.OTLP_NODE_LOG_SERVICE

    def __init__(self):
        super().__init__(OtlpNodeLogService)

    def create(self):
        return OtlpNodeLogService()



import ctypes

from common.metrology_auth.base import BaseClass


class Authorization(BaseClass):

    def __init__(self, ctype_filename):
        self.lib = self.get_lib(ctype_filename)

        class CheckReturnType(ctypes.Structure):
            _fields_ = [
                ("r0", ctypes.c_char_p),
                ("r1", ctypes.c_char_p),
                ("r2", ctypes.c_char_p),
            ]

        class CheckV2ReturnType(ctypes.Structure):
            _fields_ = [
                ("r0", ctypes.c_char_p),
                ("r1", ctypes.c_char_p),
                ("r2", ctypes.c_char_p),
            ]

        self.CheckReturnType = CheckReturnType
        self.CheckV2ReturnType = CheckV2ReturnType

        self.lib.Check.restype = self.CheckReturnType
        self.lib.Check.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.POINTER(ctypes.c_char_p),
            ctypes.c_int,
            ctypes.c_char_p,
        ]

        self.lib.CheckV2.restype = self.CheckV2ReturnType
        self.lib.CheckV2.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.POINTER(ctypes.c_char_p),
            ctypes.c_int,
            ctypes.c_char_p,
        ]

        self.lib.Check_Init.restype = ctypes.c_char_p
        self.lib.Check_Init.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_int,
            ctypes.POINTER(ctypes.c_char_p),
            ctypes.c_int,
            ctypes.c_char_p,
        ]

        self.lib.Check_Fini.restype = None
        self.lib.Check_Fini.argtypes = []

        self.check_inited = False

    def check_init(self, url, pro, gro, service, version, mode, channel_list, sname):
        url = url.encode()
        pro = pro.encode()
        gro = gro.encode()
        service = service.encode()
        version = version.encode()
        sname = sname.encode()
        channel_array = (ctypes.c_char_p * len(channel_list))(
            *[c.encode("utf-8") for c in channel_list]
        )
        result = self.lib.Check_Init(
            url,
            pro,
            gro,
            service,
            version,
            mode,
            channel_array,
            len(channel_list),
            sname,
        )
        if not result:
            self.check_inited = True
            return None
        else:
            return result.decode()

    def check(self, appid, uid, channel, func_list, tag):
        # if self.check_inited is False:
        #     raise Exception("check not inited")
        appid = appid.encode()
        uid = uid.encode()
        channel = channel.encode()
        func_array = (ctypes.c_char_p * len(func_list))(
            *[f.encode() for f in func_list]
        )
        tag = tag.encode()
        check_result = self.lib.Check(
            appid, uid, channel, func_array, len(func_list), tag
        )
        return (
            check_result.r0.decode() if check_result.r0 else None,
            check_result.r1.decode() if check_result.r1 else None,
            check_result.r2.decode() if check_result.r2 else None,
        )

    def checkV2(self, appid, uid, channel, func_list, tag):
        # if self.check_inited is False:
        #     raise Exception("check not inited")
        appid = appid.encode()
        uid = uid.encode()
        channel = channel.encode()
        func_array = (ctypes.c_char_p * len(func_list))(
            *[f.encode() for f in func_list]
        )
        tag = tag.encode()
        check_result = self.lib.CheckV2(
            appid, uid, channel, func_array, len(func_list), tag
        )
        return (
            check_result.r0.decode() if check_result.r0 else None,
            check_result.r1.decode() if check_result.r1 else None,
            check_result.r2.decode() if check_result.r2 else None,
        )

    def check_fini(self):
        self.lib.Check_Fini()



from pydantic import Field

from common.otlp.args.base import BaseOtlpArgs


class OtlpTraceArgs(BaseOtlpArgs):
    trace_timeout: int = Field(default=0)
    trace_max_queue_size: int = Field(default=0)
    trace_schedule_delay_millis: int = Field(default=0)
    trace_max_export_batch_size: int = Field(default=0)
    trace_export_timeout_millis: int = Field(default=0)



from common.service.base import ServiceType
from common.service.cache import factory as cache_factory
from common.service.db import factory as db_factory
from common.service.kafka import factory as kafka_factory
from common.service.log import factory as log_factory
from common.service.ma import factory as ma_factory
from common.service.oss import factory as oss_factory
from common.service.otlp.metric import factory as otlp_metric_factory
from common.service.otlp.node_log import factory as otlp_node_log_factory
from common.service.otlp.sid import factory as otlp_sid_factory
from common.service.otlp.span import factory as otlp_span_factory
from common.service.settings import factory as settings_factory


def get_cache_factories_and_deps() -> list:
    """get cache factories and dependencies"""
    fac_and_deps = [
        (
            cache_factory.CacheServiceFactory(),
            [ServiceType.CACHE_SERVICE],
        )
    ]
    return fac_and_deps


def get_db_factories_and_deps() -> list:
    """get db factories and dependencies"""
    fac_and_deps = [
        (
            db_factory.DatabaseServiceFactory(),
            [ServiceType.DATABASE_SERVICE],
        )
    ]
    return fac_and_deps


def get_log_factories_and_deps() -> list:
    """get log factories and dependencies"""
    fac_and_deps = [
        (
            log_factory.LogServiceFactory(),
            [ServiceType.LOG_SERVICE],
        )
    ]
    return fac_and_deps


def get_ma_factories_and_deps() -> list:
    """get ma factories and dependencies"""
    fac_and_deps = [
        (
            ma_factory.MASDKServiceFactory(),
            [ServiceType.MASDK_SERVICE],
        )
    ]
    return fac_and_deps


def get_kafka_factories_and_deps() -> list:
    """get kafka factories and dependencies"""
    fac_and_deps = [
        (
            kafka_factory.KafkaProducerServiceFactory(),
            [ServiceType.KAFKA_PRODUCER_SERVICE],
        )
    ]
    return fac_and_deps


def get_oss_factories_and_deps() -> list:

    fac_and_deps = [
        (
            oss_factory.OSSServiceFactory(),
            [ServiceType.OSS_SERVICE],
        )
    ]
    return fac_and_deps


def get_otlp_metric_factories_and_deps() -> list:

    fac_and_deps = [
        (
            otlp_metric_factory.OtlpMetricFactory(),
            [ServiceType.OTLP_METRIC_SERVICE],
        )
    ]
    return fac_and_deps


def get_otlp_span_factories_and_deps() -> list:

    fac_and_deps = [
        (
            otlp_span_factory.OtlpSpanFactory(),
            [ServiceType.OTLP_SPAN_SERVICE],
        )
    ]
    return fac_and_deps


def get_otlp_node_log_factories_and_deps() -> list:

    fac_and_deps = [
        (
            otlp_node_log_factory.OtlpNodeLogFactory(),
            [ServiceType.OTLP_NODE_LOG_SERVICE],
        )
    ]
    return fac_and_deps


def get_otlp_sid_factories_and_deps() -> list:

    fac_and_deps = [
        (
            otlp_sid_factory.OtlpSidFactory(),
            [ServiceType.OTLP_SID_SERVICE],
        )
    ]
    return fac_and_deps


def get_settings_factories_and_deps() -> list:
    """
    Get the factories and dependencies for the settings service.
    """

    fac_and_deps = [
        (
            settings_factory.SettingsServiceFactory(),
            [ServiceType.SETTINGS_SERVICE],
        )
    ]
    return fac_and_deps


service_type_methods = {
    ServiceType.CACHE_SERVICE: get_cache_factories_and_deps,
    ServiceType.DATABASE_SERVICE: get_db_factories_and_deps,
    ServiceType.LOG_SERVICE: get_log_factories_and_deps,
    ServiceType.MASDK_SERVICE: get_ma_factories_and_deps,
    ServiceType.KAFKA_PRODUCER_SERVICE: get_kafka_factories_and_deps,
    ServiceType.OSS_SERVICE: get_oss_factories_and_deps,
    ServiceType.OTLP_METRIC_SERVICE: get_otlp_metric_factories_and_deps,
    ServiceType.OTLP_NODE_LOG_SERVICE: get_otlp_node_log_factories_and_deps,
    ServiceType.OTLP_SID_SERVICE: get_otlp_sid_factories_and_deps,
    ServiceType.OTLP_SPAN_SERVICE: get_otlp_span_factories_and_deps,
    ServiceType.SETTINGS_SERVICE: get_settings_factories_and_deps,
}


def get_factories_and_deps(services: list | None = None) -> list:
    """
    Get factories and dependencies for the given services.
    """

    fac_and_deps = []
    for service in services or []:
        # print("---- service:: ", service)
        if service not in service_type_methods:
            raise ValueError(f"{service} is not a valid service")

        fac_and_deps.extend(service_type_methods[service]())

    return fac_and_deps



import os

from loguru import logger

from common.service.base import ServiceFactory
from common.service.log.logger_service import LogService


class LogServiceFactory(ServiceFactory):
    def __init__(self):
        super().__init__(LogService)

    def create(self):
        """
        Initialize log instance.
        :return:
        """
        print("------ init loguru in factory")
        current_dir = os.getenv("LOG_PATH", "./")
        log_dir = os.path.join(current_dir, "logs")
        os.makedirs(log_dir, exist_ok=True)

        log_path = os.path.join(log_dir, "app.log")
        log_level = os.getenv("LOG_LEVEL", "ERROR")

        logger.remove()

        log_format = "{level} | {time:YYYY-MM-DD HH:mm:ss} | {file} - {function}: {line} {message}"

        logger.add(
            log_path,
            rotation="100 MB",
            retention="10 days",
            compression="zip",
            format=log_format,
            serialize=True,
            level=log_level,
        )
        logger.debug(
            f"Loguru initialized successfully. Log file: {log_path}, Log level: {log_level}"
        )
        return LogService()


# print("------ logger factory")



"""
查询本机ip地址
"""

import socket


def get_host_ip():
    """
    Query local ip address
    :return: ip
    """
    ip = ""
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
        return ip


local_ip = get_host_ip()



import asyncio
from typing import Literal

from common.audit_system.audit_api.base import ContentType, Stage
from common.audit_system.base import FrameAuditResult, InputFrameAudit, OutputFrameAudit
from common.audit_system.enums import Status
from common.audit_system.strategy.base_strategy import AuditStrategy
from common.audit_system.utils import ALL_SENTENCE_LEN, Sentence
from common.exceptions.codes import c9020
from common.exceptions.errs import AuditServiceException
from common.otlp.trace.span import Span

WORKFLOW_MAX_SENTENCE_LEN = 1500


class TextAuditStrategy(AuditStrategy):
    """
    文本送审策略实现，包括帧送审和首句送审。
    """

    async def input_review(self, input_frame: InputFrameAudit, span: Span):
        """
        输入内容审核逻辑。
        :param input_frame:
        :param span:         Span对象，用于跟踪请求的上下文信息。
        :return:
        """
        if input_frame.content_type == ContentType.TEXT:
            for audit_api in self.audit_apis:
                await audit_api.input_text(
                    content=input_frame.content,
                    chat_sid=self.context.chat_sid,
                    span=span,
                    chat_app_id=self.context.chat_app_id,
                    uid=self.context.uid,
                    template_id=self.context.template_id,
                    context_list=input_frame.context_list,
                )
        else:
            raise ValueError("Unsupported content type for input review")

    async def output_review(self, output_frame: OutputFrameAudit, span: Span):
        """
        文本送审逻辑，分为首句送审和分句送审。
        :param output_frame
        :param span
        :return:
        """

        if not self.context.last_content_stage:
            self.context.last_content_stage = output_frame.stage

        if self.context.last_content_stage == output_frame.stage:
            self.context.remaining_content += output_frame.content

        self.context.add_source_content(output_frame)

        if not self.context.first_sentence_audited:
            span.add_info_event("↓↓↓↓↓↓↓↓↓↓↓ 首句送审 ↓↓↓↓↓↓↓↓↓↓↓")
            await self._first_sentence_audit(output_frame, span)

        else:
            span.add_info_event("↓↓↓↓↓↓↓↓↓↓↓ 分句送审 ↓↓↓↓↓↓↓↓↓↓↓")
            await self._sentence_audit(output_frame, span)

    async def _first_sentence_audit(self, output_frame: OutputFrameAudit, span: Span):
        """
        首句送审逻辑。

        1. 在answer和reasoning_content转变时，置相应is_stage_end参数为1
        2. 在answer和reasoning_content转变时，如果answer或者reasoning_content所有内容返回
           不包含结束性标点符号甚至也没有非结束性标点符号，则拼接前面的answer或reasoning_content内容，
           组成首句或者分句送审。该情况会存在漏审情况。如示例中的“敏感词”则会正常上屏。

        :param output_frame:
        :return:
        """

        first_sentence_conditions = (
            Sentence.has_end_symbol(self.context.remaining_content)
            or len(self.context.remaining_content) > WORKFLOW_MAX_SENTENCE_LEN
            or self.context.last_content_stage != output_frame.stage
            or output_frame.status == Status.STOP
        )

        if self.context.last_content_stage == output_frame.stage:
            fallback_length = (
                WORKFLOW_MAX_SENTENCE_LEN
                if output_frame.status != Status.STOP
                else ALL_SENTENCE_LEN
            )
        else:
            fallback_length = ALL_SENTENCE_LEN

        if self.context.frame_blocked:
            span.add_info_event("中间帧被审核，后续帧不再送审，直到获取首帧")
            if first_sentence_conditions:
                await self.__first_sentence_audit(output_frame, span, fallback_length)
            else:
                pass

        elif first_sentence_conditions:
            await self.__first_sentence_audit(output_frame, span, fallback_length)

        else:
            span.add_info_event("首句送审条件不满足，继续拼接内容，进行中间帧送审")
            frame_audit_result = FrameAuditResult(
                content=output_frame.content, source_frame=output_frame.source_frame
            )
            span.add_info_event(f"送审帧内容：{output_frame}")
            try:
                for audit_api in self.audit_apis:
                    span.add_info_event(f"当前审核API: {audit_api.audit_name}")
                    await audit_api.output_text(
                        stage=output_frame.stage,
                        content=self.context.remaining_content,
                        pindex=self.context.pindex,
                        span=span,
                        is_pending=1,
                        is_stage_end=0,
                        is_end=0,
                        chat_sid=self.context.chat_sid,
                        chat_app_id=self.context.chat_app_id,
                        uid=self.context.uid,
                    )
                self.context.frame_ids_on_screen.append(output_frame.frame_id)
                self.context.pindex += 1
                await self.context.output_queue_put(frame_audit_result, span)
            except Exception:
                self.context.frame_blocked = True

        if self.context.last_content_stage != output_frame.stage:
            self.context.last_content_stage = output_frame.stage
            await self.output_review(output_frame, span)

    async def __first_sentence_audit(
        self, output_frame: OutputFrameAudit, span: Span, fallback_length: int
    ):
        span.add_info_event("首句送审条件满足，进行首句送审")
        if output_frame.not_need_submit:
            sentences = [self.context.remaining_content]
            self.context.remaining_content = ""
        else:
            sentences, self.context.remaining_content = Sentence.find_valid_sentence(
                self.context.remaining_content, fallback_length=fallback_length
            )
        sentences = (
            [""]
            if (
                (
                    output_frame.status == Status.STOP
                    or self.context.last_content_stage != output_frame.stage
                )
                and not sentences
            )
            else sentences
        )
        span.add_info_event("分句结果如下：")
        span.add_info_events(
            {
                "sentences": sentences,
                "remaining_content": self.context.remaining_content,
            }
        )
        self.context.pindex = 1
        await self._audit_api_output_text_async(sentences, output_frame, span)
        self.context.first_sentence_audited = True

    async def _sentence_audit(self, output_frame: OutputFrameAudit, span: Span):
        """
        分句送审。
        :param output_frame:
        :return:
        """

        if (
            Sentence.has_end_symbol(self.context.remaining_content)
            or len(self.context.remaining_content) > WORKFLOW_MAX_SENTENCE_LEN
            or self.context.last_content_stage != output_frame.stage
            or output_frame.status == Status.STOP
        ):
            sentences = []

            if output_frame.not_need_submit:
                sentences = [self.context.remaining_content]
                self.context.remaining_content = ""

            elif (
                self.context.last_content_stage != output_frame.stage
                or output_frame.status == Status.STOP
            ):
                while True:
                    sentences_temp, self.context.remaining_content = (
                        Sentence.find_valid_sentence(
                            self.context.remaining_content, WORKFLOW_MAX_SENTENCE_LEN
                        )
                    )
                    sentences.extend(sentences_temp)
                    if not self.context.remaining_content:
                        break
                if not sentences:
                    sentences.append("")
            else:
                sentences, self.context.remaining_content = (
                    Sentence.find_valid_sentence(
                        self.context.remaining_content, WORKFLOW_MAX_SENTENCE_LEN
                    )
                )

            if (
                output_frame.status == Status.STOP
                and not sentences
                and not self.context.remaining_content
            ):
                sentences.append("")

            await self._audit_api_output_text_async(sentences, output_frame, span)

        if self.context.last_content_stage != output_frame.stage:
            self.context.last_content_stage = output_frame.stage
            await self.output_review(output_frame, span)

    async def _audit_api_output_text_async(
        self, sentences: list[str], output_frame: OutputFrameAudit, span: Span
    ):
        """
        异步文本输出审核API调用
        :param sentences:       分句
        :param output_frame:    当前帧信息
        :param span:
        :return:
        """
        current_status = output_frame.status
        pindex = self.context.pindex
        audit_tasks = []

        for idx, sentence in enumerate(sentences):
            if current_status == Status.STOP:
                output_frame.status = (
                    Status.STOP if idx == len(sentences) - 1 else Status.NONE
                )

            audit_tasks.append(
                asyncio.create_task(
                    self._audit_api_output_text(
                        output_frame.stage,
                        sentence,
                        span,
                        pindex + idx + 1,
                        current_status=output_frame.status,
                    ),
                )
            )
        _ = await asyncio.gather(*audit_tasks)
        self.context.pindex += len(sentences)
        self.context.audited_content += "".join(sentences)
        for _ in range(len(sentences)):
            await self.context.add_audited_content(span)

    async def _audit_api_output_text(
        self,
        current_stage: Stage,
        need_audit_content: str,
        span: Span,
        pindex: int,
        current_status: Status = Status.NONE,
    ):
        """
        文本输出审核API调用
        :param current_stage:
        :param need_audit_content:
        :param span:
        :param pindex:
        :param current_status:
        :return:
        """
        if self.context.error:
            span.add_info_event(f"审核上下文错误: {self.context.error}, 后续帧不再送审")
            return

        span.add_info_event(f"当前送审内容: {need_audit_content}")
        frame_audit_result = FrameAuditResult(
            content=need_audit_content, status=current_status
        )

        is_end: Literal[0, 1] = 0
        is_stage_end: Literal[0, 1] = 0

        if self.context.last_content_stage == current_stage:
            is_end = 1 if current_status == Status.STOP else 0

            if current_status == Status.STOP:
                is_stage_end = 1

        if self.context.last_content_stage != current_stage:
            is_stage_end = 1

        try:
            for audit_api in self.audit_apis:
                span.add_info_event(f"当前审核API: {audit_api.audit_name}")
                stage_arg = (
                    current_stage
                    if self.context.last_content_stage == current_stage
                    else self.context.last_content_stage
                )

                if stage_arg is None:
                    raise AuditServiceException(*c9020)("stage不可为空")
                await audit_api.output_text(
                    stage=stage_arg,
                    content=need_audit_content,
                    pindex=pindex,
                    span=span,
                    is_pending=0,
                    is_stage_end=is_stage_end,
                    is_end=is_end,
                    chat_sid=self.context.chat_sid,
                    chat_app_id=self.context.chat_app_id,
                    uid=self.context.uid,
                )
        except AuditServiceException as e:
            frame_audit_result.error = e
            span.add_error_event(f"审核API调用异常: {str(e)}")
            await self.context.output_queue_put(frame_audit_result, span)
        except Exception as e:
            span.add_error_event(f"审核API调用异常: {str(e)}")
            frame_audit_result.error = AuditServiceException(*c9020)(
                f"审核结果异常: {str(e)}"
            )
            await self.context.output_queue_put(frame_audit_result, span)
        finally:
            if frame_audit_result.error:
                self.context.error = frame_audit_result.error



import os

from loguru import logger

from common.service.base import ServiceFactory, ServiceType
from common.service.oss.base_oss import BaseOSSService
from common.service.oss.ifly_storage_gateway_service import IFlyGatewayStorageClient


class OSSServiceFactory(ServiceFactory):
    name = ServiceType.OSS_SERVICE

    def __init__(self):
        super().__init__(BaseOSSService)
        self.client = None

    def create(self):
        logger.debug("Creating OSS Servie")
        oss_type = os.getenv("OSS_TYPE", "ifly_gateway_storage")
        if oss_type == "s3":
            pass
        else:
            self.client = IFlyGatewayStorageClient(
                endpoint=os.getenv("OSS_ENDPOINT", ""),
                access_key_id=os.getenv("OSS_ACCESS_KEY_ID", ""),
                access_key_secret=os.getenv("OSS_ACCESS_KEY_SECRET", ""),
                bucket_name=os.getenv("OSS_BUCKET_NAME", ""),
                ttl=int(os.getenv("OSS_TTL", "86400")),
            )
        return self.client



from pydantic import Field

from common.otlp.args.base import BaseOtlpArgs


class OtlpMetricArgs(BaseOtlpArgs):
    metric_timeout: int = Field(default=0)
    metric_export_interval_millis: int = Field(default=0)
    metric_export_timeout_millis: int = Field(default=0)



SERVER_REQUEST_TOTAL = "server_request_total"
RELY_SERVER_REQUEST = "rely_server_request"
SERVER_REQUEST_TIME = "server_request_time"
SERVER_REQUEST_TIME_MICROSECONDS = "server_request_time_microseconds"

SERVER_APPID_REQUEST_TIME = "server_appid_request_time"
RELY_SERVER_REQUEST_TIME = "rely_server_request_time"
SERVER_CONC = "server_conc"
RELY_SERVER_CONC = "rely_server_conc"


SERVER_REQUEST_DESC = "服务入口错误数"
RELY_SERVER_REQUEST_DESC = "服务出口错误数"
SERVER_REQUEST_TIME_DESC = "服务入口性能"
RELY_SERVER_REQUEST_TIME_DESC = "服务出口性能"
SERVER_CONC_DESC = "服务入口并发数"
RELY_SERVER_CONC_DESC = "服务出口并发数"



import json
import os
from enum import Enum

from loguru import logger
from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.propagate import extract, inject
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import SpanLimits, TracerProvider
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    SpanExporter,
    SpanExportResult,
)
from opentelemetry.trace import StatusCode

from common.otlp.ip import local_ip


class SpanLevel(Enum):
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARN = "WARN"
    ERROR = "ERROR"


class FileSpanExporter(SpanExporter):

    def export(self, spans):
        try:
            for span in spans:
                content = f"Span: {json.dumps(json.loads(span.to_json()), ensure_ascii=False)}"

                if (
                    span.name == SpanLevel.ERROR.value
                    or span.status.status_code == StatusCode.ERROR
                ):
                    logger.error(content)
                elif span.name == SpanLevel.INFO.value:
                    logger.info(content)
                elif span.name == SpanLevel.WARN.value:
                    logger.warning(content)
                else:
                    logger.debug(content)
        except Exception as e:
            logger.error(f"Error exporting spans: {e}")
        return SpanExportResult.SUCCESS

    def shutdown(self):
        return SpanExportResult.SUCCESS


def init_trace(
    endpoint: str,
    service_name: str,
    timeout: int = 5000,
    max_queue_size: int = 2048,
    schedule_delay_millis: int = 5000,
    max_export_batch_size: int = 512,
    export_timeout_millis: int = 30000,
    span_limit: int = 1000,
):
    """
    初始化trace
    :param endpoint:        otlp endpoint
    :param service_name:    服务名称
    :param timeout:         超时时间
    :param max_queue_size:          表示BatchSpanProcessor数据导出的最大队列大小。默认值:2048
    :param schedule_delay_millis:   表示BatchSpanProcessor的两个连续导出之间的延迟间隔。默认值:5000
    :param max_export_batch_size:   表示BatchSpanProcessor数据导出的最大批处理大小。默认值:512
    :param export_timeout_millis:   表示从BatchSpanProcessor导出数据的最大允许时间。默认值:30000
    :param span_limit:      表示每个跟踪器可以跟踪的最大span数量。默认值:1000
    :return:
    """
    assert endpoint is not None, "otlp endpoint is None"
    assert service_name is not None, "service_name is None"

    span_limits = SpanLimits(max_events=span_limit)

    resource = Resource(
        attributes={
            SERVICE_NAME: service_name,
            "ip": local_ip,
            "serviceName": service_name,
        }
    )

    provider = TracerProvider(resource=resource, span_limits=span_limits)
    if os.getenv("OTLP_ENABLE", "false").lower() in (
        "true",
        "1",
        "yes",
        "on",
    ):
        exporter = OTLPSpanExporter(insecure=True, endpoint=endpoint, timeout=timeout)

        processor = BatchSpanProcessor(
            exporter,
            max_queue_size=max_queue_size,
            schedule_delay_millis=schedule_delay_millis,
            max_export_batch_size=max_export_batch_size,
            export_timeout_millis=export_timeout_millis,
        )

        provider.add_span_processor(processor)

    file_exporter = FileSpanExporter()
    file_processor = BatchSpanProcessor(file_exporter)
    provider.add_span_processor(file_processor)

    trace.set_tracer_provider(provider)
    logger.debug("trace init success")


class Trace:
    @staticmethod
    def inject_context() -> dict:
        trace_context: dict = {}
        inject(trace_context)
        return trace_context

    @staticmethod
    def extract_context(trace_context):
        return extract(trace_context)



"""
orchestrator.py
"""

from typing import TYPE_CHECKING

from common.audit_system.base import FrameAuditResult, InputFrameAudit, OutputFrameAudit
from common.audit_system.enums import Status
from common.audit_system.strategy.base_strategy import AuditStrategy

if TYPE_CHECKING:
    from common.otlp.trace.span import Span


class AuditOrchestrator:
    """
    审核统一调度器。
    """

    def __init__(self, audit_strategy: AuditStrategy):
        self.audit_strategy = audit_strategy

    async def process_output(self, output_frame: OutputFrameAudit, span: "Span"):
        """
        处理输出内容的审核逻辑。
        :param output_frame:
        :param span:
        :return:
        """
        # print(output_frame)
        if self.audit_strategy.context.error:
            raise self.audit_strategy.context.error
        with span.start(
            f"audit_orchestrator.process_output::frame_id:{output_frame.frame_id}"
        ) as context_span:
            context_span.add_info_event(f"送审帧内容：{output_frame.dict()}")

            if (
                output_frame.content == ""
                and output_frame.status != Status.STOP
                and not output_frame.none_need_audit
            ):
                context_span.add_info_event(
                    "↑↑↑↑↑↑↑↑↑↑↑ 该帧为空针，不送审 ↑↑↑↑↑↑↑↑↑↑↑"
                )
                await self.audit_strategy.context.output_queue_put(
                    FrameAuditResult(
                        content=output_frame.content,
                        status=output_frame.status,
                        source_frame=output_frame.source_frame,
                    ),
                    context_span,
                )
                return

            return await self.audit_strategy.output_review(output_frame, context_span)

    async def process_input(self, input_frame: InputFrameAudit, span: "Span"):
        """
        处理输出内容的审核逻辑。
        :param input_frame:
        :param span:
        :return:
        """
        return await self.audit_strategy.input_review(input_frame, span)



from pydantic import BaseModel, Field


class BaseOtlpArgs(BaseModel):

    inited: bool = Field(default=False)
    otlp_endpoint: str = Field(default="")
    otlp_service_name: str = Field(default="")
    otlp_dc: str = Field(default="")



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, List

from common.audit_system.audit_api.base import AuditAPI
from common.audit_system.base import AuditContext, InputFrameAudit, OutputFrameAudit

if TYPE_CHECKING:
    from common.otlp.trace.span import Span


class AuditStrategy(ABC):
    """
    抽象基类，定义审核策略的接口
    """

    def __init__(
        self,
        chat_sid: str,
        audit_apis: List[AuditAPI],
        template_id: str = "",
        chat_app_id: str = "",
        uid: str = "",
    ):
        """
        初始化审核策略。
        :param chat_sid:    会话ID，用于标识当前的聊天会话。
        :param audit_apis:  审核API列表，包含所有需要调用的审核API实例。
        :param template_id: 模板ID，用于管理控制台配置的审核策略模板，一个大模型场景使用一个模板ID，缺省则使用默认的标准策略模版。
        :param chat_app_id: 大模型应用ID，透传参数，上游分配给大模型调用方的账号标识，用于区分调用方。
        :param uid:         用户ID，透传参数，用于区分指定的用户。
        """
        self.context = AuditContext(
            chat_sid=chat_sid, template_id=template_id, chat_app_id=chat_app_id, uid=uid
        )
        self.audit_apis = audit_apis

    @abstractmethod
    async def input_review(self, input_frame: InputFrameAudit, span: "Span"):
        """
        输入内容审核逻辑，子类需要实现具体的审核逻辑。
        :param input_frame:
        :param span:         Span对象，用于跟踪请求的上下文信息。
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")

    @abstractmethod
    async def output_review(self, output_frame: OutputFrameAudit, span: "Span"):
        """
        输出内容审核逻辑，子类需要实现具体的审核逻辑。
        :param output_frame:
        :param span:
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")



import inspect
import os
import threading
import time
from typing import TYPE_CHECKING, Dict, Optional

# import atomics
from loguru import logger

from common.otlp.args import global_otlp_metric_args
from common.otlp.ip import local_ip
from common.otlp.metrics import metric

if TYPE_CHECKING:
    from common.otlp.trace.span import Span


class AtomicCounter:
    def __init__(self):
        self.value = 0
        self.lock = threading.Lock()

    def increment(self):
        with self.lock:
            self.value += 1


counter = AtomicCounter()


class Meter:
    start_time: int
    app_id: str
    in_histogram_flag = False
    func: str
    labels: Dict[str, str] = {}

    def __init__(self, app_id: str = "", func: str = ""):
        self.app_id = app_id
        self.start_time = int(int(round(time.time() * 1000)))

        if func:
            self.func = func
            return
        cf = inspect.currentframe()
        if cf is not None:
            frame = cf.f_back
            if frame is not None:
                self.func = frame.f_code.co_name
        self.labels = {}

    def set_label(self, key: str, value: str):
        self.labels[key] = value

    def _get_default_labels(self):
        default_labels = {
            "dc": global_otlp_metric_args.otlp_dc,
            "server_host": local_ip,
            "server_name": global_otlp_metric_args.otlp_service_name,
            "app_id": self.app_id,
            "func": self.func,
            "pid": os.getpid(),
        }
        if self.labels:
            default_labels.update(self.labels)
        return default_labels

    def in_error_count(
        self,
        code: int,
        lables: Optional[dict] = None,
        count: int = 1,
        is_in_histogram: bool = True,
        span: Optional["Span"] = None,
    ):
        """
        上报错误次数，默认上报耗时
        :param code:    错误码
        :param lables:  错误码标签
        :param count:   错误次数，默认为1
        :param is_in_histogram: 是否上报耗时，默认上报
        :return:
        """
        attr = self._get_default_labels()
        attr["ret"] = code

        if lables:
            attr.update(lables)

        if metric.counter is None:
            raise Exception("metric.counter is None")
        metric.counter.add(count, attr)
        counter.increment()
        if is_in_histogram:
            self.in_histogram(lables)
            self.in_histogram_flag = True
        logger.info(f"code: {code}, count: {counter.value}")
        if span:
            span.set_code(code)
        # print(f"code: {code}, count: {counter.value}, pid: {os.getpid()}")

    def in_success_count(self, lables: Optional[dict] = None, count: int = 1):
        """
        上报成功次数
        :param code:    错误码
        :param lables:  成功码标签
        :param count:   成功次数，默认为1
        :return:
        """
        self.in_error_count(0, lables, count)

    def in_histogram(self, lables: Optional[dict] = None):
        """
        上报耗时
        :param lables:  耗时标签
        :return:
        """
        if self.in_histogram_flag:
            return

        attr = self._get_default_labels()

        if lables:
            attr.update(lables)

        end_time = int(int(round(time.time() * 1000)))
        duration = end_time - self.start_time
        # print(f"duration: {duration}")
        if metric.histogram is None:
            raise Exception("histogram is None")
        metric.histogram.record(duration, attr)



from common.service.base import ServiceFactory
from common.service.settings.settings_service import SettingsService


class SettingsServiceFactory(ServiceFactory):

    def __init__(self):
        super().__init__(SettingsService)

    def create(self):
        setting_service = SettingsService()
        setting_service.sync_env_file_to_environ()
        return setting_service



"""
conc.py
"""

import ctypes

from common.metrology_auth.base import BaseClass


class Concurrent(BaseClass):

    def __init__(self, ctype_filename):
        self.lib = self.get_lib(ctype_filename)

        class ResultType(ctypes.Structure):
            _fields_ = [
                ("_pass", ctypes.c_int),
                ("addr", ctypes.c_char_p),
                ("currentUsed", ctypes.c_char_p),
                ("err", ctypes.c_char_p),
            ]

        self.ResultType = ResultType

        self.lib.Conc_Init.restype = ctypes.c_char_p
        self.lib.Conc_Init.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_int,
            ctypes.c_char_p,
            ctypes.c_char_p,
        ]

        self.lib.AcquireConc.restype = ResultType
        self.lib.AcquireConc.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
        ]

        self.lib.ReleaseConc.restype = ctypes.c_char_p
        self.lib.ReleaseConc.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
        ]

        self.conc_inited = False

    def conc_init(self, url, pro, gro, service, version, mode, addr, sname):
        url = url.encode()
        pro = pro.encode()
        gro = gro.encode()
        service = service.encode()
        version = version.encode()
        addr = addr.encode()
        sname = sname.encode()
        result = self.lib.Conc_Init(url, pro, gro, service, version, mode, addr, sname)
        if not result:
            self.conc_inited = True
            return None
        else:
            return result.decode()

    def acquire_conc(self, sid, appid, channel, function):
        sid = sid.encode()
        appid = appid.encode()
        channel = channel.encode()
        function = function.encode()
        conc_result = self.lib.AcquireConc(sid, appid, channel, function)
        return (
            conc_result._pass,
            conc_result.addr.decode() if conc_result.addr else "",
            conc_result.currentUsed.decode() if conc_result.currentUsed else "",
            conc_result.err.decode() if conc_result.err else "",
        )

    def release_conc(self, sid, appid, channel, function):
        sid = sid.encode()
        appid = appid.encode()
        channel = channel.encode()
        function = function.encode()
        conc_result = self.lib.ReleaseConc(sid, appid, channel, function)
        return conc_result.decode() if conc_result else ""



import abc
from enum import Enum
from typing import Any

from common.service.base import ServiceType


class RedisModel(Enum):
    SINGLE = 1
    CLUSTER = 2


class BaseCacheService(abc.ABC):
    """
    Abstract base class for a cache.
    """

    name = ServiceType.CACHE_SERVICE

    @abc.abstractmethod
    def get(self, key: str) -> Any:
        """
        Retrieve an item from the cache.

        Args:
            key: The key of the item to retrieve.

        Returns:
            The value associated with the key, or None if the key is not found.
        """

    @abc.abstractmethod
    def set(self, key: str, value: Any) -> None:
        """
        Add an item to the cache.

        Args:
            key: The key of the item.
            value: The value to cache.
        """

    @abc.abstractmethod
    def hash_set_ex(self, name: str, key: str, value: Any, expire_time: int) -> None:
        """
        add a hash item to the cache

        Args:
            name: the key of the item.
            key: The field of the item.
            value: The value to cache.
            expire_time: key的超时时间
        """

    @abc.abstractmethod
    def hash_get(self, name: str, key: str) -> Any:
        """
        add a hash item to the cache

        Args:
            name: the key of the item.
            key: The field of the item.
        """

    @abc.abstractmethod
    def hash_del(self, name: str, key: str) -> None:
        """
        description: 删除hash field
        """

    @abc.abstractmethod
    def hash_get_all(self, name: str) -> Any:
        """
        add a hash item to the cache

        Args:
            name: the key of the item.
        """

    @abc.abstractmethod
    def upsert(self, key: str, value: Any) -> None:
        """
        Add an item to the cache if it doesn't exist, or update it if it does.

        Args:
            key: The key of the item.
            value: The value to cache.
        """

    @abc.abstractmethod
    def delete(self, key: str) -> None:
        """
        Remove an item from the cache.

        Args:
            key: The key of the item to remove.
        """

    @abc.abstractmethod
    def clear(self) -> None:
        """
        Clear all items from the cache.
        """

    @abc.abstractmethod
    def pipeline(self) -> Any:
        """
        return redis pipe
        """

    @abc.abstractmethod
    def blpop(self, key: str, timeout: int) -> Any:
        """
        return redis blpop method
        """

    @abc.abstractmethod
    def hgetall_str(self, name: str) -> Any:
        """
        return redis str_data
        """

    @abc.abstractmethod
    def __contains__(self, key: str) -> bool:
        """
        Check if the key is in the cache.

        Args:
            key: The key of the item to check.

        Returns:
            True if the key is in the cache, False otherwise.
        """

    @abc.abstractmethod
    def __getitem__(self, key: str) -> Any:
        """
        Retrieve an item from the cache using the square bracket notation.

        Args:
            key: The key of the item to retrieve.
        """

    @abc.abstractmethod
    def __setitem__(self, key: str, value: Any) -> None:
        """
        Add an item to the cache using the square bracket notation.

        Args:
            key: The key of the item.
            value: The value to cache.
        """

    @abc.abstractmethod
    def __delitem__(self, key: str) -> None:
        """
        Remove an item from the cache using the square bracket notation.

        Args:
            key: The key of the item to remove.
        """

    def is_connected(self) -> bool:
        return True



import pickle
import re
from typing import Dict

from loguru import logger

from common.service.base import Service, ServiceType
from common.service.cache.base_cache import BaseCacheService, RedisModel


class RedisCache(BaseCacheService, Service):

    name = ServiceType.CACHE_SERVICE

    def __init__(
        self,
        addr=None,
        password=None,
        expiration_time=60 * 60,
        model: RedisModel = RedisModel.CLUSTER,
    ):
        if model == RedisModel.CLUSTER:
            self._client = self.init_redis_cluster(addr, password)
        else:
            self._client = self.init_redis(addr, password)
        logger.debug("redis init success")
        self.expiration_time = expiration_time

    def init_redis_cluster(self, cluster_addr, password):
        """
        初始化 redis 集群连接
        :param cluster_addr: 格式如下 addr1:port1,addr2:port2,addr3:port3
        :param password:
        :return:
        """
        logger.debug("redis cluster init in progress")
        from rediscluster import RedisCluster

        host_port_pairs = cluster_addr.split(",")
        cluster_nodes = []
        for pair in host_port_pairs:
            match = re.match(r"([^:]+):(\d+)", pair)
            if match:
                host = match.group(1)
                port = match.group(2)
                cluster_nodes.append({"host": host, "port": port})
        return RedisCluster(startup_nodes=cluster_nodes, password=password)

    def init_redis(self, addr: str, password: str):
        """
        初始化 redis 连接
        :param addr:
        :param password:
        :return:
        """
        logger.debug("redis init in progress")
        from redis import Redis

        host, port = addr.split(":")
        return Redis(host=host, port=int(port), password=password)

    # check connection
    def is_connected(self):
        """
        Check if the Redis client is connected.
        """
        import redis

        try:
            self._client.ping()
            return True
        except redis.exceptions.ConnectionError:
            return False

    def get(self, key):
        """
        Retrieve an item from the cache.

        Args:
            key: The key of the item to retrieve.

        Returns:
            The value associated with the key, or None if the key is not found.
        """
        value = self._client.get(key)
        return pickle.loads(value) if value else None

    def set(self, key, value):
        """
        Add an item to the cache.

        Args:
            key: The key of the item.
            value: The value to cache.
        """
        try:
            if pickled := pickle.dumps(value):
                result = self._client.setex(key, self.expiration_time, pickled)
                if not result:
                    raise ValueError("RedisCache could not set the value.")
        except TypeError as exc:
            raise TypeError(
                "RedisCache only accepts values that can be pickled. "
            ) from exc

    def hash_set_ex(self, name, key, value, expire_time):
        try:
            if pickled := pickle.dumps(value):
                result = self._client.hset(name=name, key=key, value=pickled)
                if result != 1:
                    if self._client.hexists(name=name, key=key):
                        logger.error(
                            f"update hash key {name} field {key} value {value}"
                        )
                    else:
                        logger.error(f"hash set failed, ret {result}")
                    if self._client.exists(name) and expire_time:
                        self._client.expire(name=name, time=expire_time)
                    return
                if expire_time:
                    self._client.expire(name=name, time=expire_time)
                # print(f"succeed to add name {name}, field {key} to redis")
        except TypeError as exc:
            raise TypeError(
                "RedisCache only accepts values that can be pickled. "
            ) from exc

    def hash_get(self, name, key):
        try:
            result = self._client.hget(name=name, key=key)
            # print("result: ", result)
            if result:
                return pickle.loads(result)
            else:
                return result
        except TypeError as exc:
            raise TypeError(
                "RedisCache only accepts values that can be pickled. "
            ) from exc

    def hash_del(self, name, *key):
        try:
            result = self._client.hdel(name, *key)
            need_delete = {}
            if result != len(key):
                if self._client.exists(result):
                    for field in key:
                        if self._client.hexists(name, field):
                            need_delete.update({name: field})
                            logger.error(f"failed to delete key {name} field {field}")
                        else:
                            logger.info(f"key {name} field {field} has been delete")
                else:
                    logger.info(f"key {name} has been delete")
            return result == len(key), need_delete
        except TypeError as exc:
            raise TypeError(
                "RedisCache only accepts values that can be pickled. "
            ) from exc

    def hash_get_all(self, name):
        try:
            return_dict: Dict = {}
            result: Dict = self._client.hgetall(name=name)
            if result:
                for key in result.keys():
                    key_str = key
                    if isinstance(key, bytes):
                        key_str = key.decode("utf-8")
                    return_dict.update({key_str: pickle.loads(result[key])})
                    result[key] = pickle.loads(result[key])
            # print(f"succeed to get return_dict {return_dict}")
            return return_dict
        except TypeError as exc:
            raise TypeError(
                "RedisCache only accepts values that can be pickled. "
            ) from exc

    def upsert(self, key, value):
        """
        Inserts or updates a value in the cache.
        If the existing value and the new value are both dictionaries, they are merged.

        Args:
            key: The key of the item.
            value: The value to insert or update.
        """
        existing_value = self.get(key)
        if (
            existing_value is not None
            and isinstance(existing_value, dict)
            and isinstance(value, dict)
        ):
            existing_value.update(value)
            value = existing_value

        self.set(key, value)

    def delete(self, key):
        """
        Remove an item from the cache.

        Args:
            key: The key of the item to remove.
        """
        self._client.delete(key)

    def clear(self):
        """
        Clear all items from the cache.
        """
        self._client.flushdb()

    def pipeline(self):
        """
        return redis pipe
        """
        return self._client.pipeline()

    def blpop(self, key, timeout):
        return self._client.blpop(key, timeout=timeout)

    def hgetall_str(self, name):
        result = self._client.hgetall(name)
        return {k.decode(): v.decode() for k, v in result.items()} if result else {}

    def __contains__(self, key):
        """Check if the key is in the cache."""
        return False if key is None else self._client.exists(key)

    def __getitem__(self, key):
        """Retrieve an item from the cache using the square bracket notation."""
        return self.get(key)

    def __setitem__(self, key, value):
        """Add an item to the cache using the square bracket notation."""
        self.set(key, value)

    def __delitem__(self, key):
        """Remove an item from the cache using the square bracket notation."""
        self.delete(key)

    def __repr__(self):
        """Return a string representation of the RedisCache instance."""
        return f"RedisCache(expiration_time={self.expiration_time})"



from pydantic import Field

from common.otlp.args.base import BaseOtlpArgs
from common.otlp.ip import local_ip


class OtlpSidArgs(BaseOtlpArgs):
    sid_sub: str = Field(default="svc")
    sid_location: str = Field(default="")
    sid_ip: str = Field(default=local_ip)
    sid_local_port: str = Field(default="")



from typing import Type

from common.otlp.trace.span import Span
from common.service.base import Service, ServiceType


class OtlpSpanService(Service):

    name: str = ServiceType.OTLP_SPAN_SERVICE

    def get_span(self) -> Type[Span]:
        return Span



import copy
from typing import Optional


class BaseExc(Exception):
    """

    BaseExc是xingchen-utils最基础的异常类，在使用时尽可能使用继承方式进行集成，方便对于不同类型的异常进行捕获

    例如:
    ```python

    from xingchen_utils.exceptions import BaseExc

    class AgentLLMParserExc(BaseExc):
        pass

    ```

    """

    def __init__(
        self, c: int, m: str, oc: int = 0, om: str = "", on: str = "", **kwargs: dict
    ):
        """
        c : code
        m : message
        oc : origin_code
        om : origin_message
        on : origin_name

        c代表当前系统想要抛出的错误码
        m代表当前系统想要抛出的错误码描述信息
        o_c代表当前系统调用其他系统，其他系统抛出的错误码
        o_m代表当前系统调用其他系统，其他系统抛出的错误码信息
        o_n代表当前系统调用其他系统，其他系统名称

        正常使用只需要使用 c 和 m 即可
        当需要对错误码进行映射管理的时候，可以使用 o_c o_m o_n
        """

        self.c = c
        self.m = m
        self.oc = oc
        self.om = om
        self.on = on
        self.kwargs = kwargs

    def __call__(
        self,
        am: str = "",
        *,
        c: Optional[int] = None,
        m: str = "",
        oc: Optional[int] = None,
        om: str = "",
        on: str = "",
        **kwargs: dict,
    ):
        """

        此方法会copy出一个新的异常对象便于修改，避免更改全局声明的异常

        am : append_message
        其他参数同 __init__

        当传递am时，会追加到异常声明时的m
        当传递其他和 __init__ 相同参数时，会覆盖copy出新对象的对应参数

        使用示例：
        ```python

        from xingchen_utils.exceptions import BaseExc

        class AgentLLMParserExc(BaseExc):
            pass

        err1 = AgentLLMParserExc(40021, "LLM结果解析异常")

        raise err1("大模型推理格式不正确")


        ```

        """

        n_s = copy.deepcopy(self)
        if c is not None:
            n_s.c = self.c
        if m:
            n_s.m = self.m
        if oc is not None:
            n_s.oc = self.oc
        if om:
            n_s.om = self.om
        if on:
            n_s.on = self.on

        if am:
            n_s.m = f"{n_s.m},{am}" if n_s.m else am

        n_s.kwargs = kwargs
        return n_s

    def __repr__(self):
        return f"{self.c}: {self.m}"

    def __str__(self):
        return self.__repr__()



import json
import logging
import os
import threading
import time
from typing import ClassVar, Optional, Type

from dotenv import load_dotenv
from pydantic import Field
from pydantic.fields import FieldInfo
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)

from common.service.settings.base_settings import BaseRemoteSettings

# from xingchen_utils.polaris.client import Polaris, ConfigFilter


class ProjectSettings(BaseSettings):
    # enp is "env polaris", pls is "polaris"

    hot_loading: bool = Field(default=False)
    reloading_interval: int = Field(default=300)
    model_config = SettingsConfigDict(
        env_file=os.getenv("CONFIG_ENV_PATH"), env_file_encoding="utf-8", extra="ignore"
    )

    __reload_env_file: str = ""
    _monitor_thread: Optional[threading.Thread] = None
    _running: bool = False

    remote_settings_source: ClassVar[Optional[Type[BaseRemoteSettings]]] = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.__reload_env_file = str(self.model_config.get("env_file", ""))
        if self.__reload_env_file and self.hot_loading:
            self._start_monitor()

        self.sync_fields_to_environ()

        if self.__reload_env_file:
            self.do_sync_env_file_to_environ()

    def sync_fields_to_environ(self):
        env_keys = os.environ.keys()
        for field, field_info in self.model_fields.items():
            env_key, env_value = self.dump_field_and_value(field, field_info)
            if env_key in env_keys:
                continue
            os.environ[env_key] = str(env_value)

    def do_sync_env_file_to_environ(self):
        load_dotenv(self.__reload_env_file, override=False)

    def dump_field_and_value(self, field: str, field_info: FieldInfo):
        value = getattr(self, field)
        if getattr(field_info, "json_schema_extra", None) is None:
            env_key = field
        else:
            if isinstance(field_info.json_schema_extra, dict):
                env_key = str(field_info.json_schema_extra.get("env", "")) or field
            else:
                env_key = field
        if isinstance(value, str):
            env_value = value
        if isinstance(value, set):
            env_value = json.dumps(list(value), ensure_ascii=False)
        else:
            env_value = json.dumps(value, ensure_ascii=False)
        return env_key, env_value

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:

        if cls.remote_settings_source is None:
            return (
                init_settings,
                env_settings,
                dotenv_settings,
                file_secret_settings,
            )

        return (
            init_settings,
            env_settings,
            cls.remote_settings_source(settings_cls, dotenv_settings, cls),
            dotenv_settings,
            file_secret_settings,
        )

    def _reload_settings(self) -> None:
        new_settings = self.__class__(hot_loading=False)
        for field in self.model_fields:
            setattr(self, field, getattr(new_settings, field))

    #
    def _start_monitor(self):
        """启动监控线程"""
        if self._running:
            return

        self._running = True

        self._monitor_thread = threading.Thread(
            target=self._monitor_loop, daemon=True, name="SettingsMonitorThread"
        )
        self._monitor_thread.start()

    def _monitor_loop(self):
        """监控循环"""
        while self._running:
            time.sleep(self.reloading_interval)
            try:
                self._reload_settings()
            except Exception as e:
                logging.error(f"Reload settings error: {e}")



from typing import Type

from common.otlp.log_trace.node_log import NodeLog
from common.otlp.log_trace.node_trace_log import NodeTraceLog
from common.service.base import ServiceType
from common.service.otlp.node_log.base_node_log import BaseOtlpNodeLogService


class OtlpNodeLogService(BaseOtlpNodeLogService):

    name = ServiceType.OTLP_NODE_LOG_SERVICE

    def get_node_log(self) -> Type[NodeLog]:
        return NodeLog

    def get_node_trace_log(self) -> Type[NodeTraceLog]:
        return NodeTraceLog



import os

from loguru import logger
from opentelemetry.exporter.otlp.proto.grpc.metric_exporter import OTLPMetricExporter
from opentelemetry.metrics import get_meter_provider, set_meter_provider
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader
from opentelemetry.sdk.resources import SERVICE_NAME, Resource

from common.otlp.metrics.consts import (
    SERVER_REQUEST_DESC,
    SERVER_REQUEST_TIME_DESC,
    SERVER_REQUEST_TIME_MICROSECONDS,
    SERVER_REQUEST_TOTAL,
)

counter = None
histogram = None
meter = None


def init_metric(
    endpoint: str,
    service_name: str,
    timeout: int = 5000,
    export_interval_millis: int = 1000,
    export_timeout_millis: int = 5000,
):
    """
    初始化metric
    :param endpoint:                open telemetry地址
    :param service_name:            服务名称
    :param timeout:                 默认配置 与服务端建连时间 单位ms 默认5000ms
    :param export_interval_millis:  sdk上报metric时间间隔 建议小于30000ms  默认1000ms
    :param export_timeout_millis:   默认配置 metrics上报服务端超时时间 单位ms 默认5000ms
    :return:
    """

    global counter, histogram, meter

    enable_metrics = os.getenv("OTLP_ENABLE", "true").lower() in (
        "true",
        "1",
        "yes",
        "on",
    )

    if enable_metrics:
        exporter = OTLPMetricExporter(
            insecure=True,
            endpoint=endpoint,
            timeout=timeout,
            max_export_batch_size=1000,
        )

        metric_reader = PeriodicExportingMetricReader(
            exporter,
            export_interval_millis=export_interval_millis,
            export_timeout_millis=export_timeout_millis,
        )
        metric_readers = [metric_reader]
    else:
        logger.info("Metrics reporting is disabled by environment variable")
        metric_readers = []

    assert endpoint is not None, "endpoint is None"
    assert service_name is not None, "service_name is None"

    resource = Resource(attributes={SERVICE_NAME: service_name})
    provider = MeterProvider(metric_readers=metric_readers, resource=resource)

    set_meter_provider(provider)

    meter = get_meter_provider().get_meter(f"{service_name}_meter")

    counter = meter.create_counter(
        SERVER_REQUEST_TOTAL, description=SERVER_REQUEST_DESC
    )
    histogram = meter.create_histogram(
        SERVER_REQUEST_TIME_MICROSECONDS, description=SERVER_REQUEST_TIME_DESC
    )
    logger.debug("metric init success")



from loguru import logger

from common.service import service_manager
from common.service.utils import get_factories_and_deps


def initialize_services(services: list | None = None):
    """
    Initialize all the services needed.
    """
    for factory, dependencies in get_factories_and_deps(services):
        try:
            service_manager.register_factory(factory, dependencies=dependencies)
        except Exception as exc:
            logger.exception(exc)
            raise RuntimeError(
                "Could not initialize services. Please check your settings."
            ) from exc



import os
from typing import Any, Type

from pydantic.fields import FieldInfo
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)

from common.service.settings.base_settings import (
    BaseRemoteSettings,
    BaseSettingsService,
)
from common.settings.polaris import ConfigFilter, Polaris
from common.settings.settings import ProjectSettings


class EmptyReomoteSettings(BaseRemoteSettings):

    def __init__(
        self,
        settings_cls: type[BaseSettings],
        dotenv_settings: PydanticBaseSettingsSource,
        cls: Type["ProjectSettings"],
    ):
        BaseRemoteSettings.__init__(self, settings_cls, dotenv_settings, cls)

    def get_field_value(
        self, field: FieldInfo, field_name: str
    ) -> tuple[Any, str, bool]:
        raise NotImplementedError

    def __call__(self):
        return {}


class PolarisRemoteSettings(BaseRemoteSettings):

    def __init__(
        self,
        settings_cls: type[BaseSettings],
        dotenv_settings: PydanticBaseSettingsSource,
        cls: Type["ProjectSettings"],
    ):
        BaseRemoteSettings.__init__(self, settings_cls, dotenv_settings, cls)

        self.polaris_keys = (
            "POLARIS_BASE_URL",
            "POLARIS_USERNAME",
            "POLARIS_PASSWORD",
            "POLARIS_PROJECT",
            "POLARIS_CLUSTER",
            "POLARIS_SERVICE",
            "POLARIS_VERSION",
            "POLARIS_CONFIG_FILE",
            "POLARIS_RETRY_COUNT",
            "POLARIS_RETRY_INTERVAL",
        )

    def get_field_value(
        self, field: FieldInfo, field_name: str
    ) -> tuple[Any, str, bool]:
        raise NotImplementedError

    def __call__(self) -> dict[str, Any]:

        polaris_configs = self.load_config()

        d: dict[str, Any] = {}

        for field_name, field in self.settings_cls.model_fields.items():
            field_value = polaris_configs.get(field_name, "not_set_in_polaris")

            if field_value != "not_set_in_polaris":
                d[field_name] = field_value

        return d

    def load_config(self):
        return self.load_polaris()

    def load_polaris(self):

        connect_args = []

        for key in self.polaris_keys:
            v = os.getenv(key) or self.dotenv_settings.get(key)
            if not v:
                raise ValueError(f"Polaris config {key} is missing")

            connect_args.append(v)

        polaris = Polaris(
            base_url=connect_args[0], username=connect_args[1], password=connect_args[2]
        )

        config_filter = ConfigFilter(
            project_name=connect_args[3],
            cluster_group=connect_args[4],
            service_name=connect_args[5],
            version=connect_args[6],
            config_file=connect_args[7],
        )

        content_dict = polaris.pull(
            config_filter=config_filter,
            retry_count=int(connect_args[8] or "3"),
            retry_interval=int(connect_args[9] or "5"),
            set_env=True,
        )
        return content_dict


class SettingsService(BaseSettingsService):

    @property
    def setting_base(self):
        if os.getenv("POLARIS_ENABLED", "false").lower() == "true":
            ProjectSettings.remote_settings_source = PolarisRemoteSettings
        else:
            ProjectSettings.remote_settings_source = EmptyReomoteSettings

        return ProjectSettings

    def sync_env_file_to_environ(self):
        # print("config path is:", os.getenv("CONFIG_ENV_PATH"))

        class MySettings(self.setting_base):
            model_config = SettingsConfigDict(
                env_file=os.getenv("CONFIG_ENV_PATH"),
                env_file_encoding="utf-8",
                extra="ignore",
            )

        MySettings()



import os

from common.metrology_auth import MASDK
from common.service.base import Service, ServiceType


class MASDKService(Service):
    name = ServiceType.MASDK_SERVICE

    def __init__(
        self,
        channel_list,
        strategy_type,
        polaris_url="",
        polaris_project="",
        polaris_group="",
        polaris_service="",
        polaris_version="",
        rpc_config_file=None,
        metrics_service_name=None,
    ):
        if not os.getenv("MASDK_SWITCH"):
            return
        self.ma_sdk = MASDK(
            channel_list,
            strategy_type,
            polaris_url,
            polaris_project,
            polaris_group,
            polaris_service,
            polaris_version,
            rpc_config_file,
            metrics_service_name,
        )



"""
__init__.py

"""

from typing import TYPE_CHECKING, Dict, Generator, List, Optional
from venv import logger

from sqlmodel import Session

from common.service.base import Service, ServiceFactory, ServiceType

if TYPE_CHECKING:
    from common.service.cache.base_cache import BaseCacheService
    from common.service.db.db_service import DatabaseService
    from common.service.kafka.kafka_service import KafkaProducerService
    from common.service.ma.metrology_auth_service import MASDKService
    from common.service.oss.base_oss import BaseOSSService
    from common.service.otlp.metric.base_metric import BaseOtlpMetricService
    from common.service.otlp.node_log.base_node_log import BaseOtlpNodeLogService
    from common.service.otlp.sid.sid_service import OtlpSidService
    from common.service.otlp.span.span_service import OtlpSpanService
    from common.service.settings.settings_service import SettingsService


class ServiceManager:
    """
    Service Manager
    """

    def __init__(self):
        self.services: Dict[str, "Service"] = {}
        self.factories = {}
        self.dependencies = {}

    def register_factory(
        self,
        service_factory: "ServiceFactory",
        dependencies: Optional[List[ServiceType]] = None,
    ):
        """
        Registers a new factory with dependencies.
        """
        if dependencies is None:
            dependencies = []
        service_name = service_factory.service_class.name
        self.factories[service_name] = service_factory
        self.dependencies[service_name] = dependencies
        self._create_service(service_name)

    def get(self, service_name: ServiceType):
        """
        Get (or create) a service by its name.
        """
        if service_name not in self.services:
            self._create_service(service_name)

        return self.services[service_name]

    def _create_service(self, service_name: ServiceType):
        """
        Create a new service given its name, handling dependencies.
        """
        # from loguru import logger
        logger.debug(f"Create service {service_name}")
        self._validate_service_creation(service_name)

        # Create the actual service
        self.services[service_name] = self.factories[service_name].create()
        self.services[service_name].set_ready()

    def _validate_service_creation(self, service_name: ServiceType):
        """
        Validate whether the service can be created.
        """
        if service_name not in self.factories:
            raise ValueError(
                f"No factory registered for the service class '{service_name.name}'"
            )


service_manager = ServiceManager()


def get_db_service() -> "DatabaseService":
    return service_manager.get(ServiceType.DATABASE_SERVICE)


def get_session() -> Generator["Session", None, None]:
    db_service = service_manager.get(ServiceType.DATABASE_SERVICE)
    yield from db_service.get_session()


def get_cache_service() -> "BaseCacheService":
    return service_manager.get(ServiceType.CACHE_SERVICE)


def get_kafka_producer_service() -> "KafkaProducerService":
    """获取kafka生产者服务"""
    return service_manager.get(ServiceType.KAFKA_PRODUCER_SERVICE)


def get_oss_service() -> "BaseOSSService":
    """获取oss服务"""
    return service_manager.get(ServiceType.OSS_SERVICE)


def get_masdk_service() -> "MASDKService":
    """获取metrology服务"""
    return service_manager.get(ServiceType.MASDK_SERVICE)


def get_otlp_metric_service() -> "BaseOtlpMetricService":
    """获取otlp指标服务"""
    return service_manager.get(ServiceType.OTLP_METRIC_SERVICE)


def get_otlp_span_service() -> "OtlpSpanService":
    """获取otlp span服务"""
    return service_manager.get(ServiceType.OTLP_SPAN_SERVICE)


def get_oltp_sid_service() -> "OtlpSidService":
    """获取otlp sid服务"""
    return service_manager.get(ServiceType.OTLP_SID_SERVICE)


def get_otlp_node_log_service() -> "BaseOtlpNodeLogService":
    """获取otlp node log服务"""
    return service_manager.get(ServiceType.OTLP_NODE_LOG_SERVICE)


def get_settings_service() -> "SettingsService":
    """获取设置服务"""
    return service_manager.get(ServiceType.SETTINGS_SERVICE)



from common.otlp.args.metric import OtlpMetricArgs
from common.otlp.args.sid import OtlpSidArgs
from common.otlp.args.trace import OtlpTraceArgs

global_otlp_metric_args = OtlpMetricArgs()
global_otlp_trace_args = OtlpTraceArgs()
global_otlp_sid_args = OtlpSidArgs()



import time
from contextlib import contextmanager
from datetime import datetime, timedelta
from io import StringIO
from typing import Optional

import requests
from dotenv import dotenv_values, load_dotenv
from pydantic import BaseModel, Field


class LoginPayload(BaseModel):
    addr: Optional[str] = Field(None)
    account: Optional[str] = Field(None)
    password: Optional[str] = Field(None)


class ConfigFilter(BaseModel):
    project_name: str
    cluster_group: str
    service_name: str
    version: str
    config_file: str


class Polaris(BaseModel):
    base_url: str
    username: str
    password: str
    refresh: int = 60
    cookie_create_at: datetime = Field(
        default_factory=lambda: datetime.now() - timedelta(seconds=60 + 1)
    )
    cookie: Optional[str] = Field(default="")

    def _login_payload(self):
        payload = LoginPayload(
            addr=self.base_url, account=self.username, password=self.password
        )
        return payload.model_dump(by_alias=False)

    def _set_cookie(self, session: requests.Session):
        url = f"{self.base_url}/api/v1/user/login"
        response = session.post(url, json=self._login_payload(), timeout=5)

        if response.status_code == 200:
            self.cookie = response.cookies.get_dict().get("JSESSIONID")
            self.cookie_create_at = datetime.now()

    @contextmanager
    def _session(self):
        with requests.Session() as session:
            self._set_cookie(session)
            yield session

    @staticmethod
    def set_env(configs_content: str):
        load_dotenv(stream=StringIO(configs_content), override=False)

    def pull(
        self,
        config_filter: ConfigFilter,
        retry_count: int = 3,
        retry_interval: int = 10,
        set_env=True,
        verbose=False,
    ):
        for i in range(retry_count):
            try:
                with self._session() as session:
                    url = f"{self.base_url}/config/download?project={config_filter.project_name}&cluster={config_filter.cluster_group}&service={config_filter.service_name}&version={config_filter.version}&configName={config_filter.config_file}"
                    response = session.get(url, cookies={"JSESSIONID": self.cookie})
                    response.raise_for_status()
                    content = response.json()["data"]["content"]
                    if set_env:
                        self.set_env(content)
                    if verbose:
                        print(content)
                    content_dict = dotenv_values(stream=StringIO(content))
                    return content_dict
            except Exception as e:
                if i == retry_count - 1:
                    raise e
            i += 1
            time.sleep(retry_interval)



from typing import Optional
from urllib.parse import urlencode

# import boto3
import requests
from loguru import logger

from common.exceptions.codes import c9010
from common.exceptions.errs import OssServiceException
from common.service.base import Service, ServiceType
from common.service.oss.base_oss import BaseOSSService
from common.utils.hmac_auth import HMACAuth


class IFlyGatewayStorageClient(BaseOSSService, Service):

    name = ServiceType.OSS_SERVICE

    def __init__(
        self,
        endpoint: str,
        access_key_id: str,
        access_key_secret: str,
        bucket_name: str,
        ttl: int,
    ):
        """


        Args:
            endpoint:
            access_key_id:
            access_key_secret:
            bucket_name:
            ttl:
        """
        self.endpoint = endpoint
        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret
        self.bucket_name = bucket_name
        self.ttl = ttl

    def upload_file(
        self, filename: str, file_bytes: bytes, bucket_name: Optional[str] = None
    ) -> str:

        url = f"{self.endpoint}/api/v1/{self.bucket_name}"
        params = {
            "get_link": "true",
            "link_ttl": self.ttl,
            "filename": filename,
            "expose": "true",
        }
        url = url + "?" + urlencode(params)
        headers = HMACAuth.build_auth_header(
            url,
            method="POST",
            api_key=self.access_key_id,
            api_secret=self.access_key_secret,
        )
        headers["X-TTL"] = str(self.ttl)
        headers["Content-Length"] = str(len(file_bytes))
        try:
            resp = requests.post(url, headers=headers, data=file_bytes)
        except Exception as e:
            logger.error(e)
            return ""
        if resp.status_code != 200:
            raise OssServiceException(*c9010)(
                f"invoke oss error, status_code: {resp.status_code}, message: {resp.text}"
            )

        ret = resp.json()
        if ret["code"] != 0:
            raise OssServiceException(*c9010)(
                f"invoke oss error, status_code: {resp.status_code}, message: {resp.text}"
            )
        try:
            link = ret["data"]["link"]
        except Exception:
            raise OssServiceException(*c9010)(
                f"invoke oss error, status_code: {resp.status_code}, message: {resp.text}"
            )
        return link



import os

from common.otlp.args import global_otlp_metric_args
from common.otlp.metrics.metric import init_metric
from common.service.base import ServiceFactory, ServiceType
from common.service.otlp.metric.metric_service import OtlpMetricService


def init_otlp_metric():

    # global global_otlp_metric_args

    if global_otlp_metric_args.inited:
        return

    global_otlp_metric_args.otlp_endpoint = os.getenv("OTLP_ENDPOINT", "")
    global_otlp_metric_args.otlp_service_name = os.getenv("SERVICE_NAME", "")
    global_otlp_metric_args.otlp_dc = os.getenv("SERVICE_LOCATION", "")
    global_otlp_metric_args.metric_timeout = int(
        os.getenv("OTLP_METRIC_TIMEOUT", "5000")
    )
    global_otlp_metric_args.metric_export_interval_millis = int(
        os.getenv("OTLP_METRIC_EXPORT_INTERVAL_MILLIS", "3000")
    )
    global_otlp_metric_args.metric_export_timeout_millis = int(
        os.getenv("OTLP_METRIC_EXPORT_TIMEOUT_MILLIS", "5000")
    )
    init_metric(
        global_otlp_metric_args.otlp_endpoint,
        global_otlp_metric_args.otlp_service_name,
        global_otlp_metric_args.metric_timeout,
        global_otlp_metric_args.metric_export_interval_millis,
        global_otlp_metric_args.metric_export_timeout_millis,
    )


class OtlpMetricFactory(ServiceFactory):
    name = ServiceType.OSS_SERVICE

    def __init__(self):
        super().__init__(OtlpMetricService)

    def create(self):
        init_otlp_metric()
        # if config.mode == "public":
        # return # api service
        return OtlpMetricService()



import ctypes

from common.metrology_auth.base import BaseClass


class Report(BaseClass):

    def __init__(self, ctype_filename):
        self.lib = self.get_lib(ctype_filename)

        self.lib.Report_Init.restype = ctypes.c_char_p
        self.lib.Report_Init.argtypes = [
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_char_p,
            ctypes.c_int,
            ctypes.c_char_p,
            ctypes.c_char_p,
        ]

        self.lib.Report.restype = ctypes.c_char_p
        self.lib.Report.argtypes = [
            ctypes.c_char_p,
            ctypes.POINTER(ctypes.c_char_p),
            ctypes.c_int,
            ctypes.POINTER(ctypes.c_uint),
        ]

        self.lib.Report_Fini.restype = None
        self.lib.Report_Fini.argtypes = []

        self.report_inited = False

    def report_init(self, url, pro, gro, service, version, mode, addr, sname):
        """
        初始化上报
        """
        url = url.encode()
        pro = pro.encode()
        gro = gro.encode()
        service = service.encode()
        version = version.encode()
        addr = addr.encode()
        sname = sname.encode()
        result = self.lib.Report_Init(
            url, pro, gro, service, version, mode, addr, sname
        )
        if not result:
            self.report_inited = True
            return None
        else:
            return result.decode()

    def report(self, channel, conc_info_keys, conc_info_values):
        channel = channel.encode()
        conc_info_keys_array = (ctypes.c_char_p * len(conc_info_keys))(
            *[k.encode() for k in conc_info_keys]
        )
        conc_info_values_array = (ctypes.c_uint * len(conc_info_values))(
            *conc_info_values
        )
        result = self.lib.Report(
            channel, conc_info_keys_array, len(conc_info_keys), conc_info_values_array
        )
        return result.decode() if result else None

    def report_fini(self):
        self.lib.Report_Fini()



from common.otlp import sid as sid_module
from common.service.base import Service, ServiceType


class OtlpSidService(Service):

    name: str = ServiceType.OTLP_SID_SERVICE

    def sid(self) -> str:
        if isinstance(sid_module.sid_generator2, sid_module.SidGenerator2):
            return sid_module.sid_generator2.gen()
        raise Exception("sid_generator2 is not initialized")



import os

from common.service.base import ServiceFactory
from common.service.ma.metrology_auth_service import MASDKService


class MASDKServiceFactory(ServiceFactory):
    def __init__(self):
        super().__init__(MASDKService)

    def create(self):
        polaris_url = os.getenv("MASDK_POLARIS_URL")
        if not polaris_url:
            raise ValueError("MASDK_POLARIS_URL 环境变量未配置")

        polaris_project = os.getenv("MASDK_POLARIS_PROJECT")
        if not polaris_url:
            raise ValueError("MASDK_POLARIS_URL 环境变量未配置")

        polaris_group = os.getenv("MASDK_POLARIS_GROUP")
        if not polaris_url:
            raise ValueError("MASDK_POLARIS_URL 环境变量未配置")

        polaris_service = os.getenv("MASDK_POLARIS_SERVICE")
        if not polaris_url:
            raise ValueError("MASDK_POLARIS_URL 环境变量未配置")

        polaris_version = os.getenv("MASDK_POLARIS_VERSION")
        if not polaris_url:
            raise ValueError("MASDK_POLARIS_URL 环境变量未配置")

        channel_list = [os.getenv("MASDK_CHANNEL")]
        if not polaris_url:
            raise ValueError("MASDK_POLARIS_URL 环境变量未配置")

        metrics_service_name = "masdk"
        strategy_type = ["cnt", "conc"]

        return MASDKService(
            channel_list,
            strategy_type,
            polaris_url,
            polaris_project,
            polaris_group,
            polaris_service,
            polaris_version,
            None,
            metrics_service_name,
        )



import abc
from typing import Optional

from common.service.base import ServiceType


class BaseOSSService(abc.ABC):
    """
    Abstract base class for a cache.
    """

    name = ServiceType.OSS_SERVICE

    @abc.abstractmethod
    def upload_file(
        self, filename: str, file_bytes: bytes, bucket_name: Optional[str] = None
    ) -> str:
        raise NotImplementedError



"""
base.py
"""

from abc import ABC
from enum import Enum
from typing import List


class Service(ABC):
    """
    Base class for all services.
    """

    name: str
    ready: bool = False

    def teardown(self) -> None:
        """
        Teardown the service.
        """

    def set_ready(self) -> None:
        """
        Set the service as ready.
        """
        self.ready = True


class ServiceFactory:
    """
    Base class for service factories.

    """

    def __init__(self, service_class: Service):
        self.service_class = service_class

    def create(self, *args: tuple, **kwargs: dict) -> Service:
        """
        Create a service instance.
        """
        raise NotImplementedError

    def get_service_class(self) -> Service:
        """获取服务类"""
        return self.service_class


class ServiceType(str, Enum):
    """
    Enum for the different types of services that can be
    registered with the service manager.
    """

    CACHE_SERVICE = "cache_service"
    DATABASE_SERVICE = "database_service"
    LOG_SERVICE = "log_service"
    KAFKA_PRODUCER_SERVICE = "kafka_producer_service"
    OSS_SERVICE = "oss_service"
    MASDK_SERVICE = "masdk_service"
    OTLP_METRIC_SERVICE = "otlp_metric_service"
    OTLP_NODE_LOG_SERVICE = "otlp_node_log_service"
    OTLP_SPAN_SERVICE = "otlp_span_service"
    OTLP_SID_SERVICE = "otlp_sid_service"
    SETTINGS_SERVICE = "settings_service"

    @staticmethod
    def list() -> List[str]:
        """list of service types"""
        return list(map(lambda c: c.value, ServiceType))



from abc import ABC, abstractmethod
from enum import Enum
from typing import List, Literal

from pydantic import BaseModel, Field

from common.otlp.trace.span import Span


class Stage(str, Enum):
    REASONING = "reasoning"
    ANSWER = "answer"


class ContentType(str, Enum):
    TEXT = "text"
    IMAGE = "image"
    VIDEO = "video"
    AUDIO = "audio"


class ResourceList(BaseModel):
    """
    跟问答关联的资源信息，作用于降低上下文结合风险。
    """

    data_id: str
    content_type: ContentType
    res_desc: str
    ocr_text: str


class ContextList(BaseModel):
    """
    多轮对话场景下历史对话信息，作用于降低上下文结合风险，按照交互对话顺序传递。
    """

    role: str
    content: str
    resource_list: List[ResourceList] = Field(default_factory=list)


class AuditAPI(ABC):

    audit_name: str = "BaseAuditAPI"

    @abstractmethod
    async def input_text(
        self,
        content: str,
        chat_sid: str,
        span: Span,
        chat_app_id: str = "",
        uid: str = "",
        template_id: str = "",
        context_list: List[ContextList] = [],
        **kwargs,
    ):
        """
        大模型内容安全场景中，该接口用于检测用户输入prompt内容的安全性，并针对高风险和中低风险内容提供对应的处置建议。

        :param content:         审核内容，需要送审的文本信息。
        :param chat_sid:        本轮对话ID，用于标识一次大模型对话。注意：本轮问答对话chat_sid需保持一致。
        :param span:            Span对象，用于跟踪请求的上下文信息。
        :param chat_app_id:     大模型应用ID，透传参数，上游分配给大模型调用方的账号标识，用于区分调用方。
        :param uid:             用户ID，透传参数，用于区分指定的用户。
        :param template_id:     模板ID，用于管理控制台配置的审核策略模板，一个大模型场景使用一个模板ID，缺省则使用默认的标准策略模版。
        :param context_list:    多轮对话场景下历史对话信息，作用于降低上下文结合风险，按照交互对话顺序传递。
        :param kwargs:
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")

    @abstractmethod
    async def output_text(
        self,
        stage: Stage,
        content: str,
        pindex: int,
        span: Span,
        is_pending: Literal[0, 1],
        is_stage_end: Literal[0, 1],
        is_end: Literal[0, 1],
        chat_sid: str,
        chat_app_id: str = "",
        uid: str = "",
        **kwargs,
    ):
        """
        大模型内容安全场景中，该接口用于检测大模型输出内容的安全性，并针对高风险和中低风险内容提供对应的处置建议。

        :param stage:               审核环节，用于区分大模型各使用场景下的具体环节，
                                    基于不同的环节提供更细粒度的审核控制。缺省值answer，枚举值参考附录。
        :param content:             审核内容，需要送审的文本。
        :param pindex:              输出文本片段索引，表明当前文本是第几片文本，从1开始，建议业务方按照结束标点或段落拆分片段。
        :param span:
        :param is_pending:          分片文本内容不完整标识，用于首句快速上屏等场景。0：完整分片（缺省值），1：不完整分片
        :param is_stage_end:        当前审核环节分片是否为最后分片标识，0：非最后一段（缺省值），1：最后一段
        :param is_end:              当前分片是否为最后分片标识，0：非最后一段（缺省值），1：最后一段
        :param chat_sid:            本轮对话ID，用于标识一次大模型对话。注意：本轮问答对话chat_sid需保持一致。
        :param chat_app_id:         大模型应用ID，透传参数，上游分配给大模型调用方的账号标识，用于区分调用方。
        :param uid:                 用户ID，透传参数，用于区分指定的用户。
        :param kwargs:
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")

    @abstractmethod
    async def input_media(self, text: str, **kwargs):
        """
        大模型内容安全场景中，对用户输入的文本、图片、视频、文档等进行过滤、检测和识别，并根据安全策略进行相应的处理和响应。
        :param text:
        :param kwargs:
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")

    @abstractmethod
    async def output_media(self, text: str, **kwargs):
        """
        大模型内容安全场景中，对大模型输出的图片、视频、音频等进行过滤、检测和识别，并根据安全策略进行相应的处理和响应。
        :param text:
        :param kwargs:
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")

    @abstractmethod
    async def know_ref(self, text: str, **kwargs):
        """
        大模型内容安全场景中，对大模型答复过程中引用的网站、知识库等数据进行过滤、检测和识别，并根据安全策略进行相应的处理和响应。
        :param text:
        :param kwargs:
        :return:
        """
        raise NotImplementedError("Subclasses must implement this method")



import abc
from typing import Type

from common.otlp.metrics.meter import Meter
from common.service.base import Service, ServiceType


class BaseOtlpMetricService(Service):

    name = ServiceType.OTLP_METRIC_SERVICE

    @abc.abstractmethod
    def get_meter(self) -> Type[Meter]:
        return Meter



from common.otlp.metrics.meter import Meter
from common.service.otlp.metric.base_metric import BaseOtlpMetricService


class OtlpMetricService(BaseOtlpMetricService):

    def get_meter(self):
        return Meter
